//自适应布局

var React = require("react");
var BaseUtils = require("../../utils/baseUtils");
var FormUtils = require("../../utils/formUtils");
var FormLayout = require("./formLayout");

module.exports = React.createClass({

    mixins : [FormLayout],

    compInterval: 4,

    normalHeight: "16rem",

    render: function() {
        return (
            <div className="h5-form-auto-layout">
                <ul>
                    {this.createLayoutElements()}
                </ul>
            </div>
        )
    },

    createLayoutElements: function() {
        var self = this;
        var options = this.props.options;
        var items = [];
        $.map(options.items, function(item, index) {
            var widgetName = item.widgetName;
            var elementOptions = self.createLayoutElementOptions(item);
            items.push(
                <li key={index}>
                    {FormLayout.createLayoutElement(elementOptions, index, options, widgetName, self.props.hasFormBackground)}
                </li>
            );
        });
        return items;
    },

    createLayoutElementOptions: function(item) {
        var containerWidth = BaseUtils.getWindowWidth() - 2 * this.compInterval;
        item.isAutoLayout = true;
        var options = {
            x: 0,
            y: 0,
            content: item
        }
        this.setElementStyle(item, containerWidth, options);
        return options;
    },

    setElementStyle: function(item, containerWidth, style) {
        if(!FormUtils.isWidget(item)) { //控件的宽高直接根据控件的大小调整
            style.width = containerWidth;
        }
        if(FormUtils.isChart(item)) {   //报表块的高度根据报表内容调整
            if(this.isSingleComponent()) {
                style.height = BaseUtils.getWindowHeight() - 2 * this.compInterval;
            } else if(item.width <= containerWidth || this.isPhoneTemplate()) {
                //等比例放大
                var scale = item.width / containerWidth;
                style.height = item.height / scale;
            } else {
                //图表比较宽,挤压,保持面积不变
                style.height = Math.min(item.width * item.height / containerWidth, BaseUtils.getWindowHeight() * 0.8);
            }
        }
        if(FormUtils.isTextArea(item)) {
            style.height = this.normalHeight;
        }
    },

    isPhoneTemplate: function() {
        return this.props.options.fitLayoutWidth < this.props.options.fitLayoutHeight;
    },

    isSingleComponent: function() {
        return this.props.options.items.length === 1;
    }
});

