//报表块
var React = require("react");
var ReportComponent = require("../report/reportComponents");
var BaseUtils = require("../../utils/baseUtils");
var ReportUtils = require("../../utils/reportUtils");
var ElementCaseFlip = require("./elementcaseflip");
var Widget = require("../widget/widget");
require("../../style/finereport.css");

module.exports = React.createClass({

    fitAttr: {
        "defaults": 0,
        "landscape": 1,
        "portrait": 2,
        "original": 3
    },

    landscapeFit: 1,

    portraitFit: 0,

    initialScale: 1,

    elementPadding: 2,

    mixins : [Widget],

    getInitialState: function() {
        return {
            reportContent: null,
            paramStr: null
        }
    },

    componentDidMount: function() {
        if(!this.state.reportContent) {
            this.showReportContent();
        }
        this.afterWidgetInit();
    },

    componentDidUpdate: function() {
        if(this.props.options.showToolbar) {
            this.addToolBar();
        }
    },

    render: function() {
        var reportContent = this.state.reportContent;
        if(!reportContent) {
            return <div />;
        } else {
            this.initFitAttr();
            delete reportContent.pageContent.background;    //报表块不支持纸张背景,直接用容器的
            return (
                <div className="h5-element-case" style={this.getElementCaseStyle()}>
                    <ReportComponent onDoHyperLink={this.onDoHyperLink} fireWidgetLink={this.fireWidgetLink} key={this.state.paramStr}  ref="report" reportName={this.props.options.widgetName} isElementCase={true}
                        initialScale={this.getInitialScale()} options={reportContent} refreshPageNumber={this.refreshPageNumber}
                        containerWidth={this.getContentWidth()} containerHeight={this.getContentHeight()}/>
                </div>
            );
        }
    },

    addToolBar: function() {
        var reportContent = this.state.reportContent;
        this.refreshPageNumber(reportContent.currentPageIndex, reportContent.reportTotalPage);
    },

    go2Previous: function(e) {
        this.refs["report"].go2LastPage(e);
    },

    go2Next: function(e) {
        this.refs["report"].go2NextPage(e);
    },

    refreshPageNumber: function(currentPage, totalPage) {
        this.props.options.appendTool(<ElementCaseFlip currentPage={currentPage} totalPage={totalPage} go2Previous={this.go2Previous} go2Next={this.go2Next} />);
    },

    initFitAttr: function() {
        this.landscapeFit = this.props.options.horFit;
        this.portraitFit = this.props.options.verFit;
    },

    getInitialScale: function() {
        var currentFit = this.getCurrentFit();
        var reportSize = ReportUtils.getReportSize(this.state.reportContent);
        if(currentFit === this.fitAttr.landscape) {  //横向充满
            this.initialScale = this.getContentWidth() / reportSize.width;
        } else if(currentFit === this.fitAttr.portrait) {   //纵向充满
            if(this.props.isAutoLayout) {
                this.initialScale = 1;
            } else {
                this.initialScale = this.getContentHeight() / reportSize.height;
            }
        } else if(currentFit === this.fitAttr.defaults) {   //默认
            this.initialScale = 1;
        }
        return this.initialScale;
    },

    getElementCaseStyle: function() {
        var style = {
        };
        BaseUtils.analyseBackground(this.props.options.widgetBackground, style, this.getContentWidth(), this.getContentHeight());
        if(this.props.options.border) {
            style.borderColor = this.props.options.border.color;
            style.borderadius
        }
        if(this.props.options.isMaxHeightLimit4Phone && this.props.options.maxHeightPercent4Phone > 0) {
            style["maxHeight"] = this.props.options.maxHeightPercent4Phone * BaseUtils.getWindowHeight() + "px";
        }
        return style;
    },

    getCurrentFit: function() {
        switch(window.orientation) {
            case 0:
                return this.portraitFit;
            case 90:
            case -90:
                if(this.landscapeFit === this.fitAttr.defaults) {
                    this.landscapeFit = this.fitAttr.landscape;
                }
                return this.landscapeFit;
            default:
                return this.fitAttr.defaults;
        }
    },

    showReportContent: function(parameters) {
        var self = this;
        if(!parameters) {
            parameters = {};
        }
        var options = this.props.options;
        var data = {
            op: "fr_form",
            cmd: "load_content_mobile",
            sessionID: this.props.sessionID,
            pageIndex: 1,
            widgetName: options.widgetName,
            width: this.getContentWidth(),
            height: this.getContentHeight(),
        };
        data = $.extend(true, {"__parameters__":parameters}, data);
        if (options.__parameters__) {
            data.__parameters__ = options.__parameters__;
        }
        if(data.__parameters__) {
            data.noCache = "t";
        }
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            data: data,
            dataType: 'text',
            type: 'POST',
            success: function(responseObject) {
                var jsonData = BaseUtils.jsonDecode(responseObject);
                self.setState({
                    reportContent: jsonData,
                    paramStr: JSON.stringify(parameters)
                });
            }
        });
    },

    //其他组件联动报表块
    refreshWhenBeLinked: function(parameters) {
        this.showReportContent(parameters);
    },

    //报表块联动其他组件
    fireWidgetLink: function(widgetName, parameters) {
        this.props.options.fireWidgetLink(widgetName, parameters);
    },

    onDoHyperLink: function(info) {
        this.props.options.onDoHyperLink(info);
    },

    getContentWidth: function() {
        return (this.state.width ? this.state.width : this.props.options.width) - this.elementPadding;
    },

    getContentHeight: function() {
        var height = this.state.height ? this.state.height : this.props.options.height;
        if(this.props.options.isMaxHeightLimit4Phone && this.props.options.maxHeightPercent4Phone > 0) {
            height = Math.min(height, this.props.options.maxHeightPercent4Phone * BaseUtils.getWindowHeight());
        }
        return height - this.elementPadding;
    }

});