// form组件入口
var React = require("react");
var ReactDOM = require("react-dom");
var FormUtils = require("../../utils/formUtils");
var WidgetUtils = require("../../utils/widgetUtils");
var LoadingUtils = require("../../utils/loadingUtils");
var BaseUtils = require("../../utils/baseUtils");
var ChartUtils = require("../../utils/chartUtils");
var ToolbarButton = require("../toolbar/toolbarButton");
var Toolbar = require("../toolbar/toolbar");
var ToolbarGroup = require("../toolbar/toolbarGroup");

module.exports = React.createClass({

    hasFormBackground: false,

    widgetUtils: null,  //todo:这里之前使用了全局的WidgetUtils保存联动信息,表单主题和表单参数界面之间的信息会互相覆盖

    formOptions: null,

    getInitialState: function() {
        return {
            formOptions: null,
            key: "form"
        }
    },

    componentWillMount: function() {
        this.widgetUtils = $.extend(true, {}, WidgetUtils);
    },

    componentDidMount: function() {
        window.addEventListener('resize', this.handleResize);
        this.widgetUtils.widgetsInitLink(this.refs.layout.refs);
        this.widgetUtils.widgetBindForm(this);
        if(this.hasToolBar()) {
            this.createToolBar();
        }
    },

    componentWillUnmount: function() {
        window.removeEventListener('resize', this.handleResize);
    },

    render: function() {
        var layout;
        this.formOptions = this.state.formOptions || this.props.options;
        var content = FormUtils.getBorderLayoutCenterElements($.extend(true, {}, this.formOptions));
        var isAutoLayout = FormUtils.isAutoLayout(content.appRelayout);
        content.initLink = this.initLink;
        content.fireWidgetValueChange = this.fireWidgetValueChange;
        content.fireWidgetLink = this.fireWidgetLink;
        content.onDoHyperLink = this.onDoHyperLink;
        var layoutRef = "layout";
        var style = this.initStyle(content);
        if(isAutoLayout) {
            var AutoLayout = require("./autoLayout");
            layout = <AutoLayout hasFormBackground={this.hasFormBackground} options={content} ref={layoutRef} ></AutoLayout>;
        } else {
            var OriginalLayout = require("./originalLayout");
            layout = <OriginalLayout hasFormBackground={this.hasFormBackground} options={content} ref={layoutRef} ></OriginalLayout>;
        }
        return (
            <div key={this.state.key} className="h5-form" style = {style}>
                {layout}
            </div>
        );
    },

    initLink: function(widget) {
        this.widgetUtils.initLink(widget);
    },

    fireWidgetValueChange: function(widget) {
        this.widgetUtils.fireWidgetValueChange(widget);
    },

    handleResize: function() {
        var self = this;
        if (this.timer != null) {
            clearTimeout(this.timer);
        }
        this.timer = setTimeout(function () {
            self.forceUpdate();
        }, 10);
    },

    hasToolBar: function() {
        return this.props.hasParameter;
    },

    createToolBar: function() {
        var toolbarWrap = $("<div/>").attr("id", "toolbarWrap").appendTo($("body"));
        var toolbarButtons = this.createToolBarButtons();
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this)), this.onClick);
        FRH5ReportUtils.toolBar = ReactDOM.render(<Toolbar buttons={toolbarButtons}></Toolbar>, toolbarWrap[0]);
    },

    createToolBarButtons: function() {
        var buttons = [];
        if(this.props.hasParameter) {
            buttons.push({
                key: "parameter",
                width: "100%",
                buttons: [this.createFilterButton()]
            });
        }
        return buttons;
    },

    createFilterButton: function() {
        return {
            key: "filter",
            width: "100%",
            className: "h5-toolbar-button-filter",
            onClick: this.showFilter
        }
    },

    showFilter: function() {
        FRH5ReportUtils.toolBar.hide();
        FRH5ReportUtils.beforeClick();
        FRH5ReportUtils.beforeShowParameter();
    },

    onClick:function(e){
        FRH5ReportUtils.toolBar.changeVisibility();
    },

    dbCommit: function (obj) {
        if(!this.widgetUtils.validateWidget()) {
            return ;
        }
        var sessionID = this.formOptions["sessionID"];
        var self = this;
        var __parameters__ = {
            xmlconf:obj.xmlconf,
            sessionID:sessionID,
            callback:obj.callback,
        };
        __parameters__ = $.extend(true, __parameters__, this.widgetUtils.collectWidgetValues());
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            data: {
                op: "dbcommit",
                __parameters__: __parameters__,
            },
            type: "POST",
            success: function(responseObject) {
                var response = BaseUtils.jsonDecode(responseObject);
                if(response.callback) {
                    if($.isFunction(response.callback)) {
                        response.callback.call(self);
                    } else {
                        var action = response.callback.replace(/\n/g,"");
                        (new Function("(" + action + ").apply(this, arguments)")).apply(self, arguments);
                    }
                }
            }
        });
    },

    initStyle: function(content) {
        var style = {};
        if(content.widgetBackground) {
            this.hasFormBackground = true;
            BaseUtils.analyseBackground(content.widgetBackground, style);
        }
        return style;
    },

    //form组件之间的联动
    fireWidgetLink: function(widgetName, parameters) {
        var widget = this.getWidgetByName(widgetName);
        widget.refreshWhenBeLinked(parameters);
    },

    onDoHyperLink: function(info) {
        var self = this;
        var type = info.type;
        if(type == "dynamic") {
            BaseUtils.ajax({
                url: FRH5ReportConstants.servletURL,
                dataType: "text",
                data: {
                    sessionID: info.sessionID,
                    op: "fr_form",
                    cmd: "load_content",
                },
                type: "POST",
                success: function(formResult) {
                    LoadingUtils.hideLoading();
                    ChartUtils.clear();
                    var jsonData = BaseUtils.jsonDecode(formResult);
                    jsonData.sessionID = info.sessionID;
                    self.setState({
                        formOptions: jsonData,
                        key: "refresh" + (new Date()).getTime()
                    });
                },
                error: function() {
                    LoadingUtils.hideLoading();
                    alert("failure to get form content");
                }
            });

        } else if(type == "widget") {
            this.fireWidgetLink(info.related, info.parameters);
        }
    },

    getWidgetByName: function(name) {
        return this.widgetUtils.getWidgetByName(name);
    }
});