//原样布局

var React = require("react");
var FormLayout = require("./formLayout");
var FormUtils = require("../../utils/formUtils");

module.exports = React.createClass({

    mixins : [FormLayout],

    render: function () {

        var itemGroups = this.getItemGroups();

        return (
            <div className="h5-form-original-layout">
                {this.createLayoutContents(itemGroups)}
            </div>
        );
    },

    createLayoutContents: function (itemGroups) {
        var self = this;
        var elements = [];
        var options = this.props.options;
        $.map(itemGroups, function(group, groupIndex) {
            var groupElements = [];
            $.map(group.content, function(item, itemIndex) {
                var widgetName = item.widgetName;
                var elementOptions = self.createLayoutElementOptions(item);
                groupElements.push(FormLayout.createLayoutElement(elementOptions, itemIndex, options, widgetName, self.props.hasFormBackground));
            });
            var groupContainerStyle = {
            };
            if(group.content.length > 1) {
                groupContainerStyle.height = group.height;
            }
            var groupContainer = (
                <div key={groupIndex} className="h5-form-original-layout-group" style={groupContainerStyle}>
                    {groupElements}
                </div>
            );
            elements.push(groupContainer);
        });
        return elements;
    },

    createLayoutElementOptions: function(item) {
        var elementOptions = {
            x: item.x,
            y: item.y,
            width: item.width,
            content: item,
            height: item.height
        }
        if(item.width === this.props.options.fitLayoutWidth) {
            elementOptions.isSingle = true; //单身狗
            if(FormUtils.isElementCase(item) && this.props.options.compState !== 0) {
                //原始布局中的报表块有个特殊规则， 如果单行只有一个报表块,而且不是充满展现区域,高度根据内容自适应,删除高度属性
                delete elementOptions.height;
            }
        }
        return elementOptions;
    },

    /**
     *  wei: 这里首先将表单里的元素按行分组,改动原因是来自需求:
     *  原始布局中的报表块有个特殊规则， 如果单行只有一个报表块,高度根据内容自适应,这样如果完全采用absolute布局会很麻烦,需要在报表加载结束,确定高度后再调整布局.
     *  在生成布局前,先按照行来分组,组与组之间高度会自动调整,组内的内容采用absolute布局
     */
    getItemGroups: function() {
        var self = this;
        var itemGroups = [];
        $.map(this.props.options.items, function(item, index) {
            self.addItem2Groups(item, itemGroups);
        });
        return itemGroups;
    },

    addItem2Groups: function(item, itemGroups) {
        if(!this.checkExists(item, itemGroups)) {
            var group = this.createGroup(item);
            itemGroups.push(group);
        }
    },

    checkExists: function(item, itemGroups) {
        var self = this;
        var exists = false;
        $.map(itemGroups, function(group, index) {
            if(self.isGroupHasItem(group, item)) {
                exists = true;
                return true;
            }
        });
        return exists;
    },

    createGroup: function(item) {
        var self = this;
        var group = {
            x: 0,
            y: item.y,
            width: this.props.options.fitLayoutWidth,
            height: item.height,
            content: []
        }
        this.addItem2Group(item, group);
        //创建完一个group后,需要判断下现有的哪些组件应该add到这个group里
        this.refreshGroup(group);
        return group;

    },

    addItem2Group: function(item, group) {
        var tempItem = $.extend(true, {}, item);
        tempItem.x = tempItem.x - group.x;
        tempItem.y = tempItem.y - group.y;
        group.content.push(tempItem);
    },

    refreshGroup: function(group) {
        var self = this;
        var needRefresh = false;
        $.map(this.props.options.items, function(item, index) {
            if(!self.isGroupHasItem(group, item) && self.isItemShouldAdd2Group(item, group)) {
                needRefresh = true;
                group.y = Math.min(group.y, item.y);
                group.height = group.height + Math.max(0, group.y - item.y) + Math.max(0, item.y + item.height - (group.y + group.height));
                self.addItem2Group(item, group);
            }
        });
        if(needRefresh) {
            this.refreshGroup(group);
        }
    },

    isItemShouldAdd2Group: function(item, group) {
        //因为group都是占据一整行的,所以item需要加入group的条件就是,左上角/左下角落在group内
        return (item.y >= group.y && item.y <= group.y + group.height) ||
            (item.y + item.height >= group.y && item.y + item.height <= group.y + group.height);
    },

    isGroupHasItem: function(group, item) {
        var hasItem = false;
        $.map(group.content, function(groupItem, index) {
            if(item.widgetName == groupItem.widgetName) {
                hasItem = true;
                return true;
            }
        });
        return hasItem;
    }

});