var React = require("react");
var ReactDOM = require("react-dom");
var EntryItem = require("./entryItem");
var SubDirectory = require("./subDirectory");
var BaseUtils = require("../../utils/baseUtils");
var DirectoryUtils = require("./../../utils/directoryUtils");
require("../../dist/css/h5.css");

//最外层目录树
module.exports = React.createClass({

    COOKIE_SELECTED_ID : "selectedID",

    needRemoveCookieWhenUnload: true,   //cookie主要是为了记录选中的哪个节点,返回后不至于丢失.完全退出需要removeCookie,进入报表则需要保留cookie

    subDirectory: null, //子目录树组件

    backgroundColor: {
        startR: 225,
        startG: 245,
        startB: 255
    },

    getInitialState: function () {
        return {
            selectedIndex: -1
        }
    },

    componentDidMount: function() {
        this.initCookie();
    },

    render: function() {
        var self = this;
        var items = [];
        $.map(this.props.options, function(item, index) {
                if(item.hasChildren) {
                    var className = "h5-entry-item";
                    if(self.state.selectedIndex == index) {
                        className += "-selected";
                    }else if(self.state.selectedIndex - 1 == index) {
                        className += "-before-selected";
                    }
                    items.push(
                        <li className={className} onTouchStart={self.onTouchStart} onTouchEnd={self.onTouchEnd}
                            onTouchCancel={self.onTouchCancel} onClick={self.onClick} key={index}>
                            <EntryItem icon={self.getIconClass(item)} text={item.text}></EntryItem>
                        </li>
                    );
                }
        });
        return <div className="h5-directory-tree"><ul>{items}</ul></div>;
    },

    initCookie: function() {
        var self = this;
        $(window).unload(function() {
            if(self.needRemoveCookieWhenUnload) {
                BaseUtils.removeCookie(self.COOKIE_SELECTED_ID);
            }
        });
        var selectedID = BaseUtils.getCookie(this.COOKIE_SELECTED_ID);
        if(selectedID) {
            var thisElement = $(ReactDOM.findDOMNode(this));
            $.map(this.props.options, function(item, index) {
                if(item.id == selectedID) {
                    self.chooseItemAtIndex(index, false);
                }
            });
        }
    },

    onTouchStart: function(e) {
        $(e.currentTarget).addClass("h5-entry-item-active");
    },

    onTouchEnd: function(e) {
        $(".h5-entry-item-active").removeClass("h5-entry-item-active");
    },

    onTouchCancel: function(e) {
        $(".h5-entry-item-active").removeClass("h5-entry-item-active");
    },

    onClick:function(e){
        this.chooseItemAtIndex(BaseUtils.getSelectedIndex($(e.target), "li"), true);
        e.stopPropagation();
    },

    chooseItemAtIndex: function(index, needAnimate) {
        BaseUtils.setCookie(this.COOKIE_SELECTED_ID, this.props.options[index].id);
        this.setState({
            selectedIndex: index
        });
        this.gotoSubDirectory(index);
        this.fadeIn(needAnimate);
    },

    //进入二级目录
    gotoSubDirectory:function(index){
        this.initSubDirectory(this.props.options[index], this.createSubWrap());
    },

    createSubWrap: function() {
        var thisElement = $(ReactDOM.findDOMNode(this));
        var spanWidth = $("span.h5-menu-icon").width();
        var subWrap = $("div.h5-sub-directory-wrap");
        if(subWrap.length == 0) {
            this.subDirectory = null;
            subWrap = $("<div />").addClass("h5-sub-directory-wrap h5-sub-directory-wrap-transitioning").appendTo(document.body);
        }
        return subWrap[0];
    },

    initSubDirectory: function(entry, container) {
        var childNodes = entry.ChildNodes ? entry.ChildNodes : [];
        if(!this.subDirectory) {
            this.subDirectory = ReactDOM.render(
                <SubDirectory defaultEntryID={entry.id} defaultEntryText={entry.text} defaultItems={childNodes} fadeIn={this.fadeIn} fadeOut={this.fadeOut} onEnterEntry={this.onEnterEntry} onMove={this.onMove}>
                </SubDirectory>,
            container);
        } else {
            this.subDirectory.setState({
                items:childNodes,
                entryID: entry.id,
                entryText: entry.text
            });
        }
        var selectEntry = {};
        selectEntry[entry.id] = childNodes;
        this.subDirectory.setSelectedEntry(selectEntry, entry.id, entry.text);
    },

    getIconClass: function(item) {
        var icon;
        var id= item.id;
        if(id.indexOf("0") == 0) {
            id = id.substring(1);
        }
        icon = this.props.icons[id] ? this.props.icons[id] : "e642";//默认文件夹图标
        return icon ? "icon-" + icon + "-a icon-menu-a; icon-" + icon + "-b icon-menu-b" : "";
    },

    fadeIn: function(needAnimate) {
        var subWrap = $("div.h5-sub-directory-wrap");
        var thisElement = $(ReactDOM.findDOMNode(this));
        if(!needAnimate) {
            BaseUtils.setTranslate3D(subWrap, "translate3d(0, 0, 0)");
            thisElement.css("backgroundColor", "#e1f5ff");
        } else {
            BaseUtils.setTranslate3D(subWrap, "translate3d(0, 0, 0)");
            thisElement.addClass("colorfadein").one("webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend", function() {
                thisElement.css("backgroundColor", "#e1f5ff");
                thisElement.removeClass("colorfadein");
            });
        }
    },

    fadeOut: function() {
        var self = this;
        var subWrap = $("div.h5-sub-directory-wrap");
        var thisElement = $(ReactDOM.findDOMNode(this));
        BaseUtils.setTranslate3D(subWrap, "translate3d(" + subWrap.width() + "px, 0, 0)");
        thisElement.addClass("colorfadeout").one("webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend", function() {
            thisElement.css("backgroundColor", "#ffffff");
            thisElement.removeClass("colorfadeout");
            self.setState({
                selectedIndex: -1
            });
        });
    },

    onMove: function(percent) {
        var red = Math.floor(this.backgroundColor.startR * (1 - percent) + 255 * percent);
        var green = Math.floor(this.backgroundColor.startG * (1 - percent) + 255 * percent);
        var blue = Math.floor(this.backgroundColor.startB * (1 - percent) + 255 * percent);
        $(ReactDOM.findDOMNode(this)).css("backgroundColor", "#" + red.toString(16) + green.toString(16) + blue.toString(16));
    },

    onEnterEntry: function() {
        this.needRemoveCookieWhenUnload = false;
    }

});



