var React = require("react");
var ReactDOM = require("react-dom");
var ReportUtils = require("../../utils/reportUtils");
var ReportContentTable = require("./reportContentTable");
var BaseUtils = require("../../utils/baseUtils");

module.exports = React.createClass({

    frozenWidth: 0,

    frozenHeight: 0,

    centerWrap: null,

    northWrap: null,

    westWrap: null,

    startX: 0,

    startY: 0,

    scrollDirection: -1,    // -1:未确定,0:横向,1:纵向

    startScrollTop: 0,

    startScrollLeft: 0,

    zoomScale: 1.0,

    eventLocation: "center",

    getInitialState: function () {
        return {
            content:null,
            containerWidth: null,
            containerHeight: null
        };
    },

    componentDidMount: function() {
        if(this.state.content) {
            this.refreshFrozenTables();
        }

        // $("div#container").css("overflow", "hidden");
    },

    componentDidUpdate: function() {
        this.refreshFrozenTables();
        this.centerWrap = $(ReactDOM.findDOMNode(this.refs["center"]));
        this.northWrap = $(ReactDOM.findDOMNode(this.refs["north"]));
        this.westWrap = $(ReactDOM.findDOMNode(this.refs["west"]));
    },

    render: function() {
        if(!this.state.content) {
            return <table/>;
        }
        var options = this.state.content;
        var rows = options.rows;
        var frozenRow = this.props.frozenRow;
        var frozenColumn = this.props.frozenColumn;
        this.frozenWidth = ReportUtils.getFrozenWidth(options, frozenColumn);
        this.frozenHeight = ReportUtils.getFrozenHeight(options, frozenRow);
        var reportWidth = ReportUtils.getFrozenReportTotalWidth(options);
        var reportHeight = ReportUtils.getFrozenReportTotalHeight(options);
        var centerWidth = this.getContainerWidth() - this.frozenWidth;
        var centerHeight = this.getContainerHeight() - this.frozenHeight;
        var disableAdjustRowHeight = (frozenColumn > 0);
        return (
            <table className="h5-frozen-table" cellPadding="0" cellSpacing="0">
                <colgroup>
                    <col style={{width:this.frozenWidth}}></col>
                    <col style={{width:centerWidth}}></col>
                </colgroup>
                <tbody>
                    <tr>
                        <td>
                            <div ref="corner" id="h5-frozen-corner" style={{height:this.frozenHeight, width:this.frozenWidth}}>
                                <ReportContentTable disableAdjustRowHeight={disableAdjustRowHeight} zoomScale={this.getZoomScale()} onDoHyperLink={this.props.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink} reportWidth={this.frozenWidth} reportHeight={this.frozenHeight} ref="cornerTable"></ReportContentTable>
                            </div>
                        </td>
                        <td>
                            <div ref="north" id="h5-frozen-north" style={{height:this.frozenHeight, width:centerWidth}} onTouchStart={this.onTouchStart} onTouchMove={this.onTouchMove} onTouchEnd={this.onTouchEnd}>
                                <ReportContentTable disableAdjustRowHeight={disableAdjustRowHeight} zoomScale={this.getZoomScale()} onDoHyperLink={this.props.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink} colOffset={frozenColumn} reportWidth={reportWidth - this.frozenWidth} reportHeight={this.frozenHeight} ref="northTable"></ReportContentTable>
                            </div>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <div ref="west" id="h5-frozen-west" style={{height:centerHeight, width:this.frozenWidth}} onTouchStart={this.onTouchStart} onTouchMove={this.onTouchMove} onTouchEnd={this.onTouchEnd}>
                                <ReportContentTable disableAdjustRowHeight={disableAdjustRowHeight} zoomScale={this.getZoomScale()} onDoHyperLink={this.props.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink} rowOffset={frozenRow} reportWidth={this.frozenWidth} reportHeight={reportHeight - this.frozenHeight} ref="westTable"></ReportContentTable>
                            </div>
                        </td>
                        <td>
                            <div ref="center" id="h5-frozen-center" style={{height:centerHeight, width:centerWidth}} onTouchStart={this.onTouchStart} onTouchMove={this.onTouchMove} onTouchEnd={this.onTouchEnd}>
                                <ReportContentTable disableAdjustRowHeight={disableAdjustRowHeight} zoomScale={this.getZoomScale()} onDoHyperLink={this.props.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink} rowOffset={frozenRow} colOffset={frozenColumn} reportWidth={reportWidth - this.frozenWidth} reportHeight={reportHeight - this.frozenHeight} ref="centerTable"></ReportContentTable>
                            </div>
                        </td>
                    </tr>
                </tbody>
            </table>
        );
    },

    onTouchStart: function(e) {
        this.startX = e.touches[0].pageX,
        this.startY = e.touches[0].pageY;
        this.eventLocation = this.getEventLocation(e);
        var element = this.getEventElement();
        this.startScrollTop = element.scrollTop();
        this.startScrollLeft = element.scrollLeft();
        e.preventDefault();
    },

    onTouchMove: function(e) {
        var moveEndX = e.touches[0].pageX;
        var moveEndY = e.touches[0].pageY;
        var X = moveEndX - this.startX;
        var Y = moveEndY - this.startY;
        if(this.scrollDirection < 0) {
            this.scrollDirection = Math.abs(X) > Math.abs(Y) ? 0 : 1;
        }
        if (this.scrollDirection == 0) {
            if(this.eventLocation == "west") {
                return ;
            }
            this.northWrap.scrollLeft(this.startScrollLeft - X);
            this.centerWrap.scrollLeft(this.startScrollLeft - X);
        } else if(this.scrollDirection == 1 ) {
            if(this.eventLocation == "north") {
                return ;
            }
            this.westWrap.scrollTop(this.startScrollTop - Y);
            this.centerWrap.scrollTop(this.startScrollTop - Y);
        }
        e.preventDefault();
    },

    onTouchEnd: function(e) {
        this.scrollDirection = -1;
        this.eventLocation = "center";
        e.preventDefault();
    },

    getEventLocation: function(e) {
        var center = $(ReactDOM.findDOMNode(this.refs["center"]));
        var west = $(ReactDOM.findDOMNode(this.refs["west"]));
        var north = $(ReactDOM.findDOMNode(this.refs["north"]));
        if(west.find($(e.target)).length > 0) {
            return "west";
        } else if(north.find($(e.target)).length > 0) {
            return "north";
        }
        return "center";
    },

    getEventElement: function() {
        switch(this.eventLocation) {
            case "west":
                return $(ReactDOM.findDOMNode(this.refs["west"]));
                break;
            case "north":
                return $(ReactDOM.findDOMNode(this.refs["north"]));
                break;
            default:
                return $(ReactDOM.findDOMNode(this.refs["center"]));
        }
    },

    adjustSize: function(scale) {
        //frozenReport unsupport zoom
        this.zoomScale = scale;
        var viewport = document.querySelector("meta[name=viewport]");
        var targetDpi = BaseUtils.isSupportTargetDPI() ? "target-densitydpi=device-dpi," : "";
        viewport.setAttribute("content", "width=device-width," + targetDpi + "user-scalable=no,initial-scale=" + scale + ",minimum-scale=" + scale + ",maximum-scale=" + scale);
    },

    refreshContent: function(content) {
        this.setState({
            content: content
        });
    },

    refreshFrozenTables: function() {
        this.refs["cornerTable"].setState({
            content:ReportUtils.getFrozenReportTableOptions(this.state.content, this.props.frozenRow, this.props.frozenColumn, "corner"),
            zoomScale: this.getZoomScale()
        });
        this.refs["northTable"].setState({
            content:ReportUtils.getFrozenReportTableOptions(this.state.content, this.props.frozenRow, this.props.frozenColumn, "north"),
            zoomScale: this.getZoomScale()
        });
        this.refs["westTable"].setState({
            content:ReportUtils.getFrozenReportTableOptions(this.state.content, this.props.frozenRow, this.props.frozenColumn, "west"),
            zoomScale: this.getZoomScale()
        });
        this.refs["centerTable"].setState({
            content:ReportUtils.getFrozenReportTableOptions(this.state.content, this.props.frozenRow, this.props.frozenColumn, "center"),
            zoomScale: this.getZoomScale()
        });
        var centerWidth = this.getContainerWidth() - this.frozenWidth;
        var centerHeight = this.getContainerHeight() - this.frozenHeight;
        $(ReactDOM.findDOMNode(this.refs["center"])).width(centerWidth).height(centerHeight);
        $(ReactDOM.findDOMNode(this.refs["west"])).height(centerHeight);
        $(ReactDOM.findDOMNode(this.refs["north"])).width(centerWidth);
    },

    getContainerWidth: function() {
        return this.state.containerWidth ? this.state.containerWidth : this.props.containerWidth;
    },

    getContainerHeight: function() {
        return this.state.containerHeight ? this.state.containerHeight : this.props.containerHeight;
    },

    getZoomScale: function() {
        return this.state.zoomScale ? this.state.zoomScale : this.props.zoomScale;
    }
});