/**
 * Created by Wei on 15/12/20.
 */

var React = require("react");
var ReactDOM = require("react-dom");
var ReportUtils = require("../../utils/reportUtils");
var BaseUtils = require("../../utils/baseUtils");
var ChartUtils = require("../../utils/chartUtils");
var Toolbar = require("../toolbar/toolbar");
var ReportContentTable = require("./reportContentTable");
var FrozenReportContentTable = require("./frozenTable");
var HintUtils = require("../../utils/hintUtils");

module.exports = React.createClass({
    fitAttr: {
        "defaults": 0,
        "landscape": 1,
        "portrait": 2,
        "original": 3
    },
    landscapeFitConfig: 1,
    portraitFitConfig: 0,
    currentPageIndex: 0,
    totalPage: 1,
    needUpdateContent: false,
    hasFrozen: false,
    originalColWidths: [],   //服务器传过来的原始列宽
    originalRowHeights: [],  //服务器传过来的原始行高
    scaleColWidths: [], //缩放后的列宽
    scaleRowHeights: [],    //缩放后的行高
    zoomScale: 1,   //缩放比例
    initialZoomScale: 1,    //最开始的缩放比例,记录一下,主要是为了提示用

    getDefaultProps: function() {
        return {
            initialScale: 1.0   //初始比例
        }
    },

    getInitialState: function() {
        return {
            containerWidth: null,
            containerHeight: null
        }
    },

    componentWillUnMount: function() {
        window.removeEventListener("resize", this.handleResize);
    },

    componentWillMount: function() {
        var tableOptions = ReportUtils.getReportTableOptions(this.props.options);
        this.originalColWidths = tableOptions["colWidths"];
        this.originalRowHeights = tableOptions["rowHeights"];
        this.zoomScale = this.getInitialScale();
        this.dealWithScale();
    },

    componentDidMount: function() {
        if(!this.props.isElementCase) {
            window.addEventListener("resize", this.handleResize);
            if(this.hasToolBar()) {
                this.createToolBar();
            }
        }
        this.updateContent();
    },

    componentDidUpdate: function() {
        if(this.props.isElementCase) {
            this.zoomScale = this.getInitialScale();
            this.dealWithScale();
            if(this.refs["reportTable"]) {
                this.refs["reportTable"].setState({
                    colWidths: this.scaleColWidths,
                    rowHeights: this.scaleRowHeights,
                    zoomScale: this.zoomScale
                });
            } else if(this.refs["frozenTable"]) {
                var content = this.refs["frozenTable"].state.content;
                content.colWidths = this.scaleColWidths;
                content.rowHeights = this.scaleRowHeights;
                this.refs["frozenTable"].setState({
                    content: content,
                    containerWidth: this.getContainerWidth(),
                    containerHeight: this.getContainerHeight(),
                    zoomScale: this.zoomScale
                });
            }
        }
    },

    render: function() {
        var self = this;
        var options = this.props.options;
        if(!this.currentPageIndex) {
            this.currentPageIndex = options.currentPageIndex;
        }
        this.totalPage = options.reportTotalPage;
        var pageContent = options["pageContent"];
        var contentTable;
        var reportSize = ReportUtils.getReportSize(options);
        if(this.initFrozen()) {
            contentTable = <FrozenReportContentTable zoomScale={this.zoomScale} onDoHyperLink={this.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink}
                containerWidth={this.getContainerWidth()} containerHeight={this.getContainerHeight()} ref="frozenTable" frozenRow={ReportUtils.getFrozenRow(options)}
                frozenColumn={ReportUtils.getFrozenColumn(options)}></FrozenReportContentTable>;
        } else {
            contentTable = <ReportContentTable zoomScale={this.zoomScale} onDoHyperLink={this.onDoHyperLink} fireWidgetLink={this.props.fireWidgetLink} ref="reportTable"></ReportContentTable>;
        }
        var background = pageContent["background"];
        var style = {
        };
        if(background) {
            this.initBackground(background, style);
        }
        return (<div className="h5-report" style={style}>
                    {contentTable}
               </div>);
    },

    onDoHyperLink: function(info) {
        var self = this;
        if(this.props.isElementCase) {
            this.props.onDoHyperLink(info);
        } else {
            var type = info.type;
            if(type == "dynamic") {
                BaseUtils.ajax({
                    url: FRH5ReportConstants.servletURL,
                    dataType: 'text',
                    data: {
                        op: "page_content",
                        cmd: "json",
                        sessionID: info.sessionID,
                        pn: 1
                    },
                    type: "POST",
                    success: function (responseObject) {
                        ChartUtils.clear();
                        self.refreshReportIndexAndContent(1, ReportUtils.getReportTableOptions(BaseUtils.jsonDecode(responseObject)));
                    }
                });
            }
        }
    },

    initFrozen: function() {
        this.hasFrozen = ReportUtils.hasFrozen(this.props.options);
        return this.hasFrozen;
    },

    hasToolBar: function() {
        return this.hasZoomButton() || this.totalPage > 1 || this.props.hasParameter;
    },

    hasZoomButton: function() {
        return this.props.options["fitConfig"].zoom && !this.hasFrozen;
    },

    initBackground: function(background, style) {
        if(background["background-color"]) {
            style["backgroundColor"] = background["background-color"];
        } else if(background["background-image"]){
            style["backgroundRepeat"] = background["background-repeat"];
            style["backgroundSize"] = background["background-size"];
            style["backgroundImage"] = BaseUtils.dealWithImageData(background["background-image"]);
        } else if(background["background"]) {
            style["background"] = background["background"];
        } else if(background["image"]) {
            ReportUtils.createImageStyle(BaseUtils.dealWithImageData(background["image"]), background["layout"], this.getContainerWidth(), this.getContainerHeight(), style);
        } else {
            BaseUtils.analyseBackground(background, style);
        }
    },

    updateContent: function() {
        var content = ReportUtils.getReportTableOptions(this.props.options);

        content.colWidths = this.scaleColWidths;
        content.rowHeights = this.scaleRowHeights;
        content.reportWidth = BaseUtils.getTotalLength(content.colWidths);
        content.reportHeight = BaseUtils.getTotalLength(content.rowHeights);

        if(this.refs["reportTable"]) {
            this.refs["reportTable"].setState({
                content:content
            });
        } else if(this.refs["frozenTable"]) {
            this.refs["frozenTable"].setState({
                content:content
            });
        }
    },

    createToolBar: function() {
        var toolbarWrap = $("<div/>").attr("id", "toolbarWrap").appendTo($("body"));
        var toolbarButtons = this.createToolBarButtons();
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this)), this.onClick);
        FRH5ReportUtils.toolBar = ReactDOM.render(<Toolbar buttons={toolbarButtons}></Toolbar>, toolbarWrap[0]);
    },

    onClick:function(e) {
        FRH5ReportUtils.toolBar.changeVisibility();
    },

    handleResize: function() {
        var self = this;
        if (this.timer != null) {
            clearTimeout(this.timer);
        }
        this.timer = setTimeout(function () {
            self.zoomScale = self.getInitialScale();
            self.dealWithScale();
            if(self.refs["reportTable"]) {
                self.refs["reportTable"].setState({
                    colWidths: self.scaleColWidths,
                    rowHeights: self.scaleRowHeights,
                    zoomScale: self.zoomScale
                });
            } else if(self.refs["frozenTable"]) {
                var content = self.refs["frozenTable"].state.content;
                content.colWidths = self.scaleColWidths;
                content.rowHeights = self.scaleRowHeights;
                self.refs["frozenTable"].setState({
                    content: content,
                    containerWidth: self.getContainerWidth(),
                    containerHeight: self.getContainerHeight(),
                    zoomScale: self.zoomScale
                });
            }
        }, 10);
    },

    createToolBarButtons: function() {
        var buttons = [];
        var buttonGroupNumbers = (this.totalPage > 1 ? 1 : 0) + (this.props.hasParameter ? 1 : 0) + (this.hasZoomButton() ? 1 : 0);
        if(buttonGroupNumbers > 0) {
            var eachGroupWidth = Math.floor((1 / buttonGroupNumbers) * 100) + "%";
            if(this.totalPage > 1) {
                buttons.push(
                    {
                        key: "page",
                        width: eachGroupWidth,
                        buttons: [this.createTurnLastPageButton(), this.createPageButton(), this.createTurnNextPageButton()]
                    }
                );
            }
            if(this.props.hasParameter) {
                buttons.push(
                    {
                        key: "parameter",
                        width: eachGroupWidth,
                        buttons: [this.createFilterButton()]

                    }
                );
            }
            if(this.hasZoomButton()) {
                buttons.push(
                    {
                        key: "zoom",
                        width: eachGroupWidth,
                        buttons: [this.createZoomOutButton(), this.createZoomInButton()]
                    }
                );
            }
        }
        return buttons;
    },

    createTurnLastPageButton: function() {
        return {
            key: "last",
            width: "30%",
            className: "h5-toolbar-button-last",
            onClick: this.go2LastPage,
            disable: true
        }
    },

    createTurnNextPageButton: function() {
        return {
            key: "next",
            width: "30%",
            className: "h5-toolbar-button-next",
            onClick: this.go2NextPage
        }
    },

    createPageButton: function() {
        return {
            key: "page",
            width: "40%",
            className: "h5-toolbar-button-page",
            text: this.currentPageIndex + "/" + this.totalPage,
            noActive: true
        }
    },

    createBlank: function(width) {
        return {
            width: width,
            className: "h5-toolbar-button-blank"
        }
    },

    createFilterButton: function() {
        return {
            key: "filter",
            width: "100%",
            className: "h5-toolbar-button-filter",
            onClick: this.showFilter
        }
    },

    createZoomInButton: function() {
        return {
            key: "zoomin",
            width: "50%",
            className: "h5-toolbar-button-zoom-in",
            onClick: this.zoomIn
        }
    },

    createZoomOutButton: function() {
        return {
            key: "zoomout",
            width: "50%",
            className: "h5-toolbar-button-zoom-out",
            onClick: this.zoomOut
        }
    },

    loadReportContent: function(sessionID, pageIndex) {
        var self = this;
        if(this.props.isElementCase) {
            ReportUtils.loadElementCaseContent(sessionID, pageIndex, this.props.reportName, function(reportContent) {
                self.refreshReportIndexAndContent(pageIndex, reportContent);
            });
        } else {
            ReportUtils.loadReportContent(sessionID, pageIndex, function(reportContent) {
                FRH5ReportUtils.toolBar.show();
                self.refreshReportIndexAndContent(pageIndex, reportContent);
            });
        }
    },

    refreshReportIndexAndContent: function(pageIndex, reportContent) {
        this.currentPageIndex = pageIndex;
        if(this.totalPage > 1) {
            if(FRH5ReportUtils.toolBar && FRH5ReportUtils.toolBar.refs["page"]) {
                FRH5ReportUtils.toolBar.refs["page"].refs["last"].setEnable((this.currentPageIndex > 1));
                FRH5ReportUtils.toolBar.refs["page"].refs["next"].setEnable((this.currentPageIndex < this.totalPage));
            }
            this.refreshPageNumber();
        }
        this.originalColWidths = reportContent["colWidths"];
        this.originalRowHeights = reportContent["rowHeights"];
        this.zoomScale = this.getInitialScale();
        this.dealWithScale();
        reportContent.colWidths = this.scaleColWidths;
        reportContent.rowHeights = this.scaleRowHeights;
        reportContent.reportWidth = BaseUtils.getTotalLength(reportContent.colWidths);
        reportContent.reportHeight = BaseUtils.getTotalLength(reportContent.rowHeights);

        if(this.refs["frozenTable"]) {
            this.refs["frozenTable"].refreshContent(reportContent);
        } else {
            this.refs["reportTable"].refreshContent(reportContent);
        }
    },

    go2LastPage: function(e) {
        e.stopPropagation();
        if(FRH5ReportUtils.toolBar) {
            FRH5ReportUtils.toolBar.refreshHideTimer();
        }
        if(this.currentPageIndex - 1 < 1) {
            return ;
        }
        this.loadReportContent(this.props.options["sessionID"], this.currentPageIndex - 1);
    },

    go2NextPage: function(e) {
        e.stopPropagation();
        if(FRH5ReportUtils.toolBar) {
            FRH5ReportUtils.toolBar.refreshHideTimer();
        }
        if(this.currentPageIndex + 1 > this.totalPage) {
            return ;
        }
        this.loadReportContent(this.props.options["sessionID"], this.currentPageIndex + 1);
    },

    refreshPageNumber: function() {
        if(this.props.isElementCase) {
            this.props.refreshPageNumber(this.currentPageIndex, this.totalPage);
        } else {
            $(".h5-toolbar-button-page").text(this.currentPageIndex + "/" + this.totalPage);
        }
    },

    showFilter: function() {
        FRH5ReportUtils.toolBar.hide();
        FRH5ReportUtils.beforeClick();
        FRH5ReportUtils.beforeShowParameter();
    },

    zoomIn: function(e) {
        e.stopPropagation();
        if(FRH5ReportUtils.toolBar) {
            FRH5ReportUtils.toolBar.refreshHideTimer();
        }
        this.zoomScale = this.zoomScale + 0.2;
        this.dealWithScale();
        this.showZoomHint();
        if(this.refs["reportTable"]) {
            this.refs["reportTable"].setState({
                colWidths: this.scaleColWidths,
                rowHeights: this.scaleRowHeights,
                zoomScale: this.zoomScale
            });
        }
    },

    zoomOut: function(e) {
        e.stopPropagation();
        if(!this.canZoomOut()) {
            return ;
        }
        if(FRH5ReportUtils.toolBar) {
            FRH5ReportUtils.toolBar.refreshHideTimer();
        }
        this.zoomScale = this.zoomScale - 0.2;
        this.dealWithScale();
        this.showZoomHint();
        if(this.refs["reportTable"]) {
            this.refs["reportTable"].setState({
                colWidths: this.scaleColWidths,
                rowHeights: this.scaleRowHeights,
                zoomScale: this.zoomScale
            });
        }
    },

    showZoomHint: function() {
        FRH5ReportUtils.toolBar.refs["zoom"].refs["zoomout"].setEnable(this.canZoomOut());
        HintUtils.showHint(Math.round((this.zoomScale - this.initialZoomScale + 1) * 100) + "%");
    },

    getCurrentFitConfig: function() {
        var fitAttribute = this.props.options["fitConfig"];
        this.landscapeFitConfig = fitAttribute ? fitAttribute["horFit"] : this.fitAttr.defaults;
        this.portraitFitConfig = fitAttribute ? fitAttribute["verFit"] : this.fitAttr.defaults;
        switch(window.orientation) {
            case 0:
                return this.portraitFitConfig;
                break;
            case 90:
            case -90:
                if(this.landscapeFitConfig === this.fitAttr.defaults) {
                    this.landscapeFitConfig = this.fitAttr.landscape;
                }
                return this.landscapeFitConfig;
                break;
            default:
                return this.fitAttr.defaults;
                break;
        }
    },

    //这个scale始终是相对于原始列宽,行高的scale
    getInitialScale: function() {
        if(this.props.isElementCase) {  //报表块还不支持放大缩小,只有一个初始比例
            this.initialZoomScale = this.props.initialScale;
        } else {
            var fitConfig = this.getCurrentFitConfig();
            var scale = 1.0;
            var reportWidth = BaseUtils.getTotalLength(this.originalColWidths);
            var reportHeight = BaseUtils.getTotalLength(this.originalRowHeights);
            var containerWidth = this.getContainerWidth();
            var containerHeight = this.getContainerHeight();
            switch(fitConfig) {
                case this.fitAttr.landscape:
                    scale = containerWidth / reportWidth;
                    break;
                case this.fitAttr.portrait:
                    scale = containerHeight / reportHeight;
                    break;
                case this.fitAttr.original:
                    scale = 1.0;
                    break;
                default:
                    break;
            }
            this.initialZoomScale = scale;
        }
        return this.initialZoomScale;
    },

    dealWithScale: function() {
        this.scaleColWidths = BaseUtils.scaleLength(this.originalColWidths, this.zoomScale);
        this.scaleRowHeights = BaseUtils.scaleLength(this.originalRowHeights, this.zoomScale);
    },

    getContainerWidth: function() {
        return this.props.isElementCase ? (this.state.containerWidth ? this.state.containerWidth : this.props.containerWidth) : BaseUtils.getWindowWidth();
    },

    getContainerHeight: function() {
        return this.props.isElementCase ? (this.state.containerHeight ? this.state.containerHeight : this.props.containerHeight) : BaseUtils.getWindowHeight();
    },

    canZoomOut: function() {
        var zoomScale = Math.round((this.zoomScale - this.initialZoomScale + 1) * 100);
        return zoomScale > 20;
    }
});

