var React = require("react");
var ReactDOM = require("react-dom");
var ReportUtils = require("../../utils/reportUtils");
var HyperLinkUtils = require("../../utils/hyperLinkUtils");
var BaseUtils = require("../../utils/baseUtils");

module.exports = React.createClass({
    getInitialState: function () {
        return {
            content:null,
            colWidths: null,
            rowHeights: null,
            zoomScale: null
        };
    },

    getDefaultProps: function() {
        return {
            rowOffset: 0,
            colOffset: 0
        }
    },

    componentDidMount: function() {
        ReportUtils.registerContentPane(this);  //support FR
    },

    render: function() {
        if(!this.state.content) {
            return <table/>;
        }
        var options = this.state.content;
        var colWidths = this.state.colWidths ? this.state.colWidths : options.colWidths;
        var rowHeights = this.state.rowHeights ? this.state.rowHeights : options.rowHeights;
        var rows = options.rows;
        var colArray = [];
        var rowArray = [];
        for(var i = 0; i < colWidths.length; i++) {
            if(colWidths[i] === 0) {
                this.recordColShouldHide(i);
            } else {
                colArray.push(React.createElement("col", {style:{width: colWidths[i]}, key:i}));
            }
        }
        for(var rowIndex = 0; rowIndex < rows.length; rowIndex++){
            var tdArray = [];
            var colNumber = 0;  //已经创建了td的col数量
            var cells = rows[rowIndex]["cells"];
            var hasShownElement = false;
            for(var cellIndex = 0; cellIndex < cells.length; cellIndex++){
                var cellData = cells[cellIndex];
                var cellCol = cellData.col;
                var cellRow = cellData.row;
                if(this.isAlreadyHaveCellAtPosition(cellRow, cellCol)) {
                    colNumber += cellData.colSpan;
                    continue ;
                }
                this.recordsCellCreate(cellData.row, cellData.col, cellData.rowSpan, cellData.colSpan);
                //后台为了减少数据传输量,只传了有数据的格子,需要补全
                if(cellCol > colNumber) {
                    for(var index = colNumber; index < cellCol; index ++) {
                        if(this.checkNeedEmptyTD(rowIndex, index)) {
                            tdArray.push(this.createEmptyTD(index));
                        }
                    }
                    colNumber = cellCol;
                }
                var style = cellData.style ? cellData.style : {};
                style.shouldHide = this.shouldHide(cellData, rowHeights, colWidths);
                if(!style.shouldHide) {
                    hasShownElement = true;
                }
                var tempCellData = $.extend(true, {}, cellData);
                this.preProcessCell4HideCol(tempCellData);
                var cellOption = {
                    text: tempCellData.text,
                    style: style,
                    colSpan: tempCellData.colSpan,
                    rowSpan: tempCellData.rowSpan,
                    value: tempCellData.value,
                    showAsHtml: tempCellData.showAsHtml,
                    hyperlink: tempCellData.hyperlink,
                    sessionID: options["sessionID"],
                    width: this.getCellWidth(tempCellData.col - this.props.colOffset, tempCellData.colSpan, colWidths),
                    height: this.getCellHeight(tempCellData.row - this.props.rowOffset, tempCellData.rowSpan, rowHeights)
                };
                tdArray.push(<ReportCell colSpanMax={colWidths.length} rowSpanMax={rowHeights.length} disableAdjustRowHeight={this.props.disableAdjustRowHeight} zoomScale={this.getZoomScale()} key={colNumber} options={cellOption} onDoHyperLink={this.onDoHyperLink}/>);
                colNumber += cellData.colSpan;
            }
            if((tdArray.length === 0 || !hasShownElement) && ReportUtils.isNeedAppendBlankTD()) {
                tdArray.push(<td key="blankTD"/>);
            }
            rowArray.push(React.createElement("tr", {style:{height: rowHeights[rowIndex]}, key:rowIndex}, tdArray));
        }
        this.clearRecordsOfCellCreate();
        return (
            <table className="h5-table" cellSpacing="0" cellPadding="0" style={{width:BaseUtils.getTotalLength(colWidths)}}>
                <colgroup key="colgroup">{colArray}</colgroup>
                <tbody key="tbody">{rowArray}</tbody>
            </table>
        );
    },

    //列宽为0的,就是需要隐藏的列,这里做下记录.
    recordColShouldHide: function(col) {
        if(!this.colShouldHide) {
            this.colShouldHide = [];
        }
        this.colShouldHide.push(col);
    },

    //列col是否隐藏
    isColHide: function(col) {
        return this.colShouldHide && this.colShouldHide.indexOf(col) >= 0;
    },

    //后台传过来的cellData json,如果做模板时先合并单元格,再隐藏其中的几列,在cellData中是反映不出来的,需要计算下.
    numberOfHideColInCell: function(col, colSpan) {
        var hideColNumber = 0;
        for(var index = 0; index < colSpan; index ++) {
            if(this.isColHide(col + index)) {
                hideColNumber ++ ;
            }
        }
        return hideColNumber;
    },

    //最终的cellData colSpan需要减去上面计算出的包含的隐藏列个数.
    preProcessCell4HideCol: function(cellData) {
        var numberOfHideColInCell = this.numberOfHideColInCell(cellData.col, cellData.colSpan);
        if(numberOfHideColInCell !== 0) {
            cellData.colSpan -= numberOfHideColInCell;
        }
    },

    //因为后台为了减少数据传输量,不传递没有内容格子的原因,需要在恰当的地方补充空td,这需要知道哪些位置已经有td占据了（主要是因为合并单元格的存在,有的行的rowSpan并不是1）
    recordsCellCreate: function(rowStart, colStart, rowSpan, colSpan) {
        if(!this.recordsForCellCreate) {
            this.recordsForCellCreate = [];
        }
        for(var rowIndex = 0; rowIndex < rowSpan; rowIndex ++) {
            for(var colIndex = 0; colIndex < colSpan; colIndex ++) {
                if(!this.recordsForCellCreate[rowIndex + rowStart]) {
                    this.recordsForCellCreate[rowIndex + rowStart] = [];
                }
                this.recordsForCellCreate[rowIndex + rowStart][colIndex + colStart] = true;
            }
        }
    },

    isAlreadyHaveCellAtPosition: function(rowIndex, colIndex) {
        return this.recordsForCellCreate && this.recordsForCellCreate[rowIndex] && this.recordsForCellCreate[rowIndex][colIndex] === true;
    },

    clearRecordsOfCellCreate: function() {
        this.recordsForCellCreate = [];
    },

    checkNeedEmptyTD: function(row, col) {
        return !this.recordsForCellCreate[row] || this.recordsForCellCreate[row][col] !== true;
    },

    createEmptyTD: function(col) {
        var style = {};
        if(this.isColHide(col)) {
            style.display = "none";
        }
        return <td style={style} key={col} colSpan="1" />;
    },

    getCellWidth: function(startCol, length, colWidths) {
        var width = 0;
        for(var i = 0; i < length; i ++) {
            width += colWidths[startCol + i];
        }
        return width;
    },

    getCellHeight: function(startRow, length, rowHeights) {
        var height = 0;
        for(var i = 0; i < length; i ++) {
            height += rowHeights[startRow + i];
        }
        return height;
    },

    shouldHide: function(cellData, rowHeights, colWidths) {
        var rowSpan = cellData.rowSpan;
        var colSpan = cellData.colSpan;
        var row = cellData.row - this.props.rowOffset;
        var col = cellData.col - this.props.colOffset;
        var rowHide = true, colHide = true;
        for(var rowIndex = 0; rowIndex < rowSpan; rowIndex ++) {
            if(rowHeights[rowIndex + row] !== 0) {
                rowHide = false;
                break;
            }
        }
        for(var colIndex = 0; colIndex < colSpan; colIndex ++) {
            if(colWidths[colIndex + col] !== 0) {
                colHide = false;
                break;
            }
        }
        return rowHide || colHide;
    },

    onDoHyperLink: function(info) {
        this.props.onDoHyperLink(info);
    },

    refreshContent: function(content) {
        this.setState({
            content: content,
            colWidths: content.colWidths,
            rowHeights: content.rowHeights
        });
    },

    getSessionID: function() {
        return this.state.content["sessionID"];
    },

    getZoomScale: function() {
        return this.state.zoomScale ? this.state.zoomScale : this.props.zoomScale;
    }
});

/**
 * 报表格子
 * props:text, style, colSpan, rowSpan, value, hyperlink
 */
var ReportCell = React.createClass({

    componentDidUpdate: function() {
        if(this.props.disableAdjustRowHeight) {
            var height = this.props.options.height - this.getBorderHeight();
            $("div.h5-cell-text-container", $(ReactDOM.findDOMNode(this))).css({
                height: height + "px",
                lineHeight: height + "px"
            });
        }
    },

    componentDidMount: function() {
        var self = this;
        var options = this.props.options;
        if(options.hyperlink) {
            BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this)), HyperLinkUtils.createHyperLinkFunction(options.hyperlink, options["sessionID"], self.props.onDoHyperLink));
        }
        if(options.showAsHtml) {
            $(".h5-cell-text-container", $(ReactDOM.findDOMNode(this))).append(options.text);
        }
        setTimeout(function() {
            //TODO: didMount立刻调用元素的offsetHeight/clientHeight属性,仍然不准确
            self.forceUpdate();
        }, 100);
    },

    getDefaultProps: function() {
        return {
            colSpan: "1",
            rowSpan: "1"
        };
    },

    propTypes: {
        text: React.PropTypes.string
    },

    render: function() {
        var options = this.props.options;
        if(!options) {
            return <td />;
        }
        var cellOption = {
            rowSpan: Math.min(options.rowSpan, this.props.rowSpanMax),
            colSpan: Math.min(options.colSpan, this.props.colSpanMax)
        };
        if(options.style) {
            cellOption["style"] = ReportUtils.createCssStyleFromCellStyle(options.style, this.props.zoomScale);
            var borderClassName = options.style.borderCss ? options.style.borderCss : "";
            var contentClassName = options.style.contentCss ? options.style.contentCss : "";
            cellOption["className"] = borderClassName + " " + contentClassName;
        }
        var className = (this.props.disableAdjustRowHeight ? "h5-cell-text-container-frozen" : "") + " h5-cell-text-container";
        var cellHeight = options.height;
        return (
            <td {...cellOption}>
                <div className={className}>
                    {this.dealWithWrapLine(options.text)}
                </div>
                {ReportUtils.createCellValue(options.value, options["sessionID"], options.width, cellHeight, this.props.onDoHyperLink)}
            </td>
        );
    },

    //处理文字中的换行
    dealWithWrapLine: function(text) {
        if(this.props.options.showAsHtml) {
            return null;
        }
        var result = [];
        if(text) {
            var textArray = text.split("\n");
            $.map(textArray, function(text, index) {
                result.push(<span key={index}>{text}</span>);
                if(index < textArray.length - 1) {
                    result.push(<br key={"br"+index}/>);
                }
            });
        }
        return result;
    },

    getBorderHeight: function() {
        if(this.borderHeight === undefined) {
            var tdElement = ReactDOM.findDOMNode(this);
            this.borderHeight = tdElement.offsetHeight - tdElement.clientHeight;
        }
        return this.borderHeight;
    }
});