var React = require("react");
var ReactDOM = require("react-dom");
var Widget = require("./widget");
var BaseUtils = require("../../utils/baseUtils");
var HyperLinkUtils = require("../../utils/hyperLinkUtils");
var ChartUtils = require("../../utils/chartUtils");

//只支持vanCharts
/**
 * 图表组件
 * props:
 *   chartWidth: 图表的宽度
 *   chartHeight: 图表的高度
 *   options: 图表的配置属性（可能符合vancharts标准,也可能不符合.不符合的需要转换下）
 */
module.exports = React.createClass({

    mixins: [Widget],

    getInitialState: function() {
        return {
            vanChartsOptions: null
        }
    },

    getDefaultProps: function() {
        return {
        }
    },

    componentDidMount: function() {
        if(this.needLoadVanCharts) {
            this.refreshVanCharts();
        }
        this.afterWidgetInit();
    },

    componentDidUpdate: function() {
        if(this.needLoadVanCharts) {
            this.refreshVanCharts();
        } else {
            this.vanCharts.resize();
        }
    },

    chartOptions: null,

    needLoadVanCharts: true,

    vanCharts: null,

    render: function() {
        this.chartOptions = this.state.vanChartsOptions || this.props.options;
        if(this.isCprImage()) {
            this.needLoadVanCharts = false;
            return this.getCprImage();
        }
        if(!this.chartOptions.simpleChartInShowID) {
            //表单里的vancharts并没有在第一次就返回可用的数据,还需要额外发请求获取,已提代码质量给eason
            this.getVanChartsData();
            return <div />;
        } else {
            var style = {
                width: this.props.chartWidth,
                height: this.props.chartHeight,
                position: "relative"
            }
            var widgetBackground = this.props.options.widgetBackground;
            if(widgetBackground) {
                BaseUtils.analyseBackground(widgetBackground, style, this.props.chartWidth, this.props.chartHeight);
            } else if(!this.props.options.hasFormBackground) {
                style.backgroundColor = "#FFF";
            }
            $.extend(true, style, this.getBorderStyle());
            if(this.props.options.border) {
                style.borderColor = this.props.options.border.color;
            }
            return (
                <div className="h5-chart" id={this.chartOptions.simpleChartInShowID} style={style}>
                </div>
            );
        }
    },

    //cpr里会把图表处理成图片
    isCprImage: function() {
        return this.chartOptions.name && this.chartOptions.url && !this.chartOptions.simpleChartInShowID;
    },

    getCprImage: function() {
        return <img style={{width: this.props.chartWidth, height: this.props.chartHeight}} src={"data:image/png;base64," + this.chartOptions.url}></img>
    },

    getVanChartsData: function() {
        var self = this;
        var options = this.props.options;
        var data = {
            op: "fr_form",
            cmd: "form_getsource",
            sessionID: options.sessionID,
            __chartsourcename__:options.widgetName,
            __chartsize__: {
                width: options.width,
                height: options.height
            }
        };
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            data: data,
            dataType: "text",
            type: "POST",
            success: function (res) {
                var jsonData = BaseUtils.jsonDecode(res);
                self.setState({
                    vanChartsOptions: jsonData.items[0]
                });
            }
        });
    },

    dealWithChartData: function(parameterStr) {
        var result = {};
        if(parameterStr.indexOf("?") === 0) {
            parameterStr = parameterStr.substring(1);
        }
        var parameterArray = parameterStr.split("&");
        $.map(parameterArray, function(item, index) {
            var itemArray = item.split("=");
            if(itemArray.length === 2) {
                result[itemArray[0]] = itemArray[1];
            }
        });
        return result;
    },

    refreshVanCharts: function() {
        var self = this;
        var options = this.chartOptions;
        if(!options.url) {
            return;
        }
        var onDoHyperLink = function(info) {
            self.props.options.onDoHyperLink(info);
        }
        var data = this.dealWithChartData(options.url);
        var sessionID = options.sessionID ? options.sessionID : data.sessionID;
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            data: data,
            dataType: "text",
            type: "POST",
            success: function (res) {
                var jsonData = BaseUtils.jsonDecode(res);
                require(['../../lib/frmethods'], function (FR) {
                    if (!window.FR) {
                        window.FR = {};
                    }
                    window.FR = $.extend(true, window.FR, FR);
                    self.vanCharts = VanCharts.init(document.getElementById(options.simpleChartInShowID));
                    if (!jsonData.chartAttr.tools) {
                        jsonData.chartAttr.tools = {};
                    }
                    jsonData.chartAttr.tools.enabled = false;
                    jsonData.chartAttr.plotOptions.click = function (e) {
                        if(this.jsonHyperlink) {
                            HyperLinkUtils.createHyperLinkFunction(this.jsonHyperlink, sessionID, onDoHyperLink)(e);
                        }
                    }
                    self.vanCharts.plotOptionsClick = jsonData.chartAttr.plotOptions.click;
                    ChartUtils.addChart(options.simpleChartInShowID, self.vanCharts);
                    self.needLoadVanCharts = false;
                    ChartUtils.drawChart(self.vanCharts, jsonData.chartAttr, false);
                });
            }
        });
    },

    //其他组件联动图表
    refreshWhenBeLinked: function(parameters) {
        var self = this;
        if(!parameters) {
            parameters = {};
        }
        parameters = $.extend(true, parameters, {
            op: "chartlink",
            cmd: "refresh_relate_data",
            sessionID: this.props.options.sessionID,
            chartWidth: this.props.chartWidth,
            chartHeight: this.props.chartHeight,
            chartID: this.chartOptions.simpleChartInShowID
        });
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            dataType: "text",
            type: "POST",
            data: parameters,
            success: function(res) {
                var jsonData = BaseUtils.jsonDecode(res);
                jsonData.relateChartList[0].chartAttr.plotOptions.click = self.vanCharts.plotOptionsClick;
                ChartUtils.drawChart(self.vanCharts, jsonData.relateChartList[0].chartAttr, true);
            }
        });
    }
});