var React = require("react");
var ReactDOM = require("react-dom");
var Widget = require("./widget");
var TriggerEditor = require("./trigger/triggerEditor");
var DateUtils = require("../../lib/jquery.date");
var BaseUtils = require("../../utils/baseUtils");

module.exports = React.createClass({

    VIEWMODE: {
        YM: 0,
        YMD: 1,
        HMS: 2,
        YMDHMS: 3
    },

    mixins: [Widget, TriggerEditor],

    DateTimePicker: null,

    datePick: null,

    componentDidMount: function () {
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.combo)), this.createDatePick);
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.arrow)), this.createDatePick);
        this.makeTriggerVisible();
        this.afterWidgetInit();
    },

    componentDidUpdate: function componentDidUpdate() {
        this.$view = $(ReactDOM.findDOMNode(this.refs["datePick"]));
        this.$view.width(this.$view.parent().width());
    },

    getInitialState: function getInitialState() {
        return {
            isShown: false,
            date: null
        };
    },

    render: function render() {
        var options = this.props.options;
        var format = options.format ? options.format : "yyyy-MM-dd";
        var dateTime = this._initDateValue(options);
        var dateStr;
        if(this.state.date) {
            dateStr = this.state.date;
        } else {
            dateStr = DateUtils.date2Str(dateTime, format);
            this.state.date = dateStr;
        }
        var attr = this.getAttrFromOptions(options);
        if (this.DateTimePicker) {
            options.viewMode = this._initViewMode();
            var minViewMode = "days";
            var viewMode = "days";
            var showDatePicker = true;
            var showTimePicker = true;
            if(options.viewMode == this.VIEWMODE.YM) {
                minViewMode = "months";
                viewMode = "months";
                showTimePicker = false;
            } else if(options.viewMode == this.VIEWMODE.HMS) {
                showDatePicker = false;
            } else if(options.viewMode == this.VIEWMODE.YMD) {
                showTimePicker = false;
            }
            var DateTimePicker = this.DateTimePicker;
            if(!this.state.isShown) {
                this.unbindTriggerEvent();
            }
            this.datePick = this.state.isShown ? (<DateTimePicker
                ref="datePick"
                style={{display: "block", position: "absolute", top: "auto", left: "auto", zIndex:6666}}
                format={this.convert2AmazeUIFormat(format)}
                onSelect={this.handleSelect}
                dateTime={dateStr}
                showTimePicker={showTimePicker}
                showDatePicker={showDatePicker}
                minViewMode={minViewMode}
                viewMode={viewMode}
                caretDisplayed={false}>
            </DateTimePicker>) : null;
        }
        return (<div className="h5-date">
                    <div ref="combo" className="combo" {...attr}>{dateStr}</div>
                    <div ref="arrow" className="arrow"></div>
                    {this.datePick}
               </div>);
    },

    convert2AmazeUIFormat: function convert2AmazeUIFormat(format) {
        var amazeUIFormat = format.replace(/y/g, "Y");
        amazeUIFormat = amazeUIFormat.replace(/d/g, "D");
        amazeUIFormat = amazeUIFormat.replace(/h/g, "H");
        amazeUIFormat = amazeUIFormat.replace(/:ss/g, "");
        return amazeUIFormat;
    },

    _initDateValue: function _initDateValue(options) {
        var date = new Date();
        if (this.state.date) {
            date = DateUtils.str2Date(this.state.date, this.props.options.format);
        } else if (options.value) {
            if(options.value.date_milliseconds) {
                date = new Date(options.value.date_milliseconds);
            } else {
                date = new Date(options.value);
            }
        }
        if(date.toString() == "Invalid Date") {
            date = new Date();
        }
        return date;
    },

    _initViewMode: function(){
        var options = this.props.options;
        var viewMode = 1;//年月日
        if (options.format.match(/[YyDd]/) && options.format.match(/[Hh]/)) {
            viewMode = 3;//年月日时分秒
        } else if (options.format.match(/[M]/) && !options.format.match(/[Dd]/) && !options.format.match(/[Hh]/)) {
            viewMode = 0;//年月
        } else if (!options.format.match(/[YyMDd]/)) {
            viewMode = 2;//时分秒
        }
        return viewMode;
    },

    collapse: function (e) {
        if(e.target.className.indexOf("am-") == 0) {
            return ;
        }
        this.setState({
            isShown: false
        });
    },

    createDatePick: function createDatePick(e) {
        if(this.props.options.disabled) {
            return ;
        }
        var self = this;
        require(["amazeui-react", "../../style/amazeui.min.css"], function(AmazeUI) {
            self.bindTriggerEvent();
            self.DateTimePicker = AmazeUI.DateTimePicker;
            self.setState({
                isShown: !self.state.isShown
            });
        });
    },

    handleSelect: function handleSelect(date) {
        var shouldSelect = !this.datePick.props.showTimePicker;
        this.state.date = date;
        this.setState({
            isShown: !shouldSelect,
            date: date
        });
        this.fireWidgetValueChange();
    },

    getValue: function getValue() {
        if (this.props.options.returnDate) {
            return DateUtils.str2Date(this.state.date, this.props.options.format);
        }
        return this.state.date;
    }
});