/**
 * Created by Wei on 15/12/28.
 */

var React = require("react");
var ReactDOM = require("react-dom");
var Widget = require("../widget");
var ComboList = require("./comboList");
var ComboItem = require("./comboItem");
var Trigger = require("./triggerEditor");
var BaseUtils = require("../../../utils/baseUtils");

module.exports = React.createClass({

    mixins: [Widget, Trigger],

    hasChosenAll: false,

    getInitialState: function() {
        return {
            data: [],
            selectedIndexes: []
        }
    },

    componentDidMount: function () {
        this.makeTriggerVisible();
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.combo)), this.showList);
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.arrow)), this.showList);
        this.afterWidgetInit();
    },

    componentWillMount: function() {
        var self = this;
        var options = this.props.options;
        if(options.value) {
            if(options.controlAttr && options.controlAttr.data) {
                var startSymbol = options.returnArray ? "" : options.startSymbol;
                var endSymbol = options.returnArray ? "" : options.endSymbol;
                var value_array = BaseUtils.toArray(options.controlAttr.value, options.delimiter, startSymbol, endSymbol);
                var selectedIndexes = [];
                $.map(options.controlAttr.data, function(item, index) {
                    if(BaseUtils.arrayContains(value_array, item["value"]) || BaseUtils.arrayContains(value_array, item["text"])) {
                        selectedIndexes.push(index + 1);    //全选/不选
                    }
                });
                if(selectedIndexes.length > 0 && selectedIndexes.length === options.controlAttr.data.length) {
                    selectedIndexes.unshift(0);
                }
                self.updateData(options.controlAttr.data, selectedIndexes);
            }
        }
    },

    render: function(){
        var options = this.props.options;
        var delimiter = options.delimiter ? options.delimiter : ",";
        var showValue = this.getSelectedData(false).join(delimiter);
        var attr = this.getAttrFromOptions(options);
        return (<div ref="wrap" className="h5-check">
                <div className="combo" {...attr} ref="combo">{showValue}</div>
                <div className="arrow" ref="arrow"></div>
                {<ComboList createItem={this.createItem} ref="comboList" checkLoad={this.checkLoad} data={this.state.data}></ComboList>}
            </div>
        );
    },

    createItem: function(index, text) {
        return <ComboItem type="check" onClick={this.didItemClick} isSelected={this.state.selectedIndexes.indexOf(index) >= 0} text={text} key={index}/>;
    },

    updateData: function (data, selectedIndexes) {
        if(!this.hasChosenAll) {
            data.unshift({text:"全选/不选", value:""});
            this.hasChosenAll = true;
        }
        var newData = {
            data: data
        }
        if(selectedIndexes) {
            newData.selectedIndexes = selectedIndexes;
        }
        this.setState(newData);
    },

    didItemClick: function(e) {
        BaseUtils.stopEvent(e);
        var item = $(e.target);
        if(item[0].nodeName.toLocaleLowerCase() != "a") {
            item = item.parent();
        }
        var items = item.parent().children();
        var index = items.indexOf(item[0]);
        var tempSelectedIndexes = this.state.selectedIndexes;
        if(index === 0) {
            //全选/全不选
            if(tempSelectedIndexes.length < this.state.data.length) {
                //全选
                tempSelectedIndexes = [];
                $.map(this.state.data, function(data, index) {
                    tempSelectedIndexes.push(index);
                })
            } else {
                //全不选
                tempSelectedIndexes = [];
            }
        } else {
            if(item.hasClass("selected")) {
                if(tempSelectedIndexes.indexOf(index) >= 0) {
                    tempSelectedIndexes.splice(tempSelectedIndexes.indexOf(index), 1);
                }
                if(tempSelectedIndexes.indexOf(0) >= 0) {
                    tempSelectedIndexes.splice(0, 1);
                }
            } else {
                tempSelectedIndexes.push(index);
                if(tempSelectedIndexes.length >= this.state.data.length - 1 && tempSelectedIndexes.indexOf(0) < 0) {
                    tempSelectedIndexes.unshift(0);
                }
            }
        }
        this.updateData(this.state.data, tempSelectedIndexes);
        e.stopPropagation();
        this.fireWidgetValueChange();
        this.fireEvent("afteredit", e);
    },

    collapse: function() {
        this.unbindTriggerEvent();
        $(ReactDOM.findDOMNode(this.refs["comboList"])).hide();
    },

    getSelectedData: function(isValue) {
        var key = isValue ? "value" : "text";
        var selectedData = [];
        for(var i = 0, len = this.state.selectedIndexes.length; i < len; i ++) {
            if(this.state.selectedIndexes[i] === 0) {
                //0是全选/非全选
                continue ;
            }
            selectedData.push(this.state.data[this.state.selectedIndexes[i]][key]);
        }
        return selectedData;
    },

    getValue: function() {
        if(this.state.selectedIndexes.length == 0) {
            return "";
        }
        var options = this.props.options;
        var selectedData = this.getSelectedData(true);
        if(options.returnArray) {
            return selectedData;
        } else {
            var delimiter = options.delimiter ? options.delimiter : ",";
            return selectedData.join(delimiter);
        }
    },

    setValue: function(value) {
        var options = this.props.options;
        var startSymbol = options.returnArray ? "" : options.startSymbol;
        var endSymbol = options.returnArray ? "" : options.endSymbol;
        var value_array = [];
        if(options.controlAttr && options.controlAttr.value) {
            value_array = BaseUtils.toArray(options.controlAttr.value, options.delimiter, startSymbol, endSymbol);
        }
        var selectedIndexes = [];
        $.map(this.state.data, function(item, index) {
            if(index > 0 && (BaseUtils.arrayContains(value_array, item["value"]) || BaseUtils.arrayContains(value_array, item["text"]))) {
                selectedIndexes.push(index);
            }
        });
        this.updateData(this.state.data, selectedIndexes);
    },

    reset: function() {
        this.hasChosenAll = false;
        this.updateData([], []);
    },

    needLoadData: function() {
        return this.state.data.length <= 1;
    }
});