var React = require("react");
var ReactDOM = require("react-dom");
var Widget = require("../../widget");
var Trigger = require("../triggerEditor");
var TreeNode = require("./treeNode");
var ComboList = require("../comboList");
var BaseUtils = require("../../../../utils/baseUtils");
var TreeNodeModel = require("./treeNodeModel");

/**
  下拉树控件
  this.state.data:视图上所有可选的节点;
 */
module.exports = React.createClass({
    mixins: [Widget, Trigger],

    baseClass: "h5-tree",

    delimiter: ",",

    getInitialState: function () {
        return {
            data: []
        };
    },

    componentDidMount: function () {
        this.makeTriggerVisible();
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.combo)), this.showList);
        BaseUtils.createClickEvent($(ReactDOM.findDOMNode(this.refs.arrow)), this.showList);
        this.afterWidgetInit();
        this.refreshComboText();
    },

    componentWillMount: function() {
        var self = this;
        var options = this.props.options;
        this.initVariables();
        if(options.controlAttr && options.controlAttr.data) {
            this.initTreeData(options.controlAttr.data);
            this.setState({
                data: options.controlAttr.data
            });
            var valueArray = (options.controlAttr.value + "").split(";");
            $.map(valueArray, function(value) {
                var treeNodeModel = self.getTreeNodeModelValue(value);
                if(!treeNodeModel) {
                    return ;
                }
                self.selectedIDs.push(treeNodeModel.getID());
                if(treeNodeModel.hasChildren && self.isSelectLeafOnly()) {
                    self.loadAllLeafNode(treeNodeModel, self.selectedIDs);
                }
                if(self.isSelectLeafOnly()) {
                    self.checkSelectUp(treeNodeModel);
                }
            });
            this.sortTreeNodeIDs(this.selectedIDs);
        }
    },

    render: function() {
        var options = this.props.options;
        var attr = this.getAttrFromOptions(options);
        var className = this.baseClass + (this.isMultiSelection() ? (" " + this.baseClass + "-multi") : "");
        return (
            <div className={className}>
                <div className="combo" {...attr} ref="combo"></div>
                <div className="arrow" ref="arrow"></div>
                {<ComboList noNeedScrollLoad={true} createItem={this.createItem} ref="comboList" checkLoad={this.checkLoad} data={this.state.data}></ComboList>}
            </div>
        );
    },

    initVariables: function() {
        this.treeNodeModels = [];
        this.treeNodeModelIDs = [];
        this.selectedIDs = [];
        this.dataCache = {};
    },

    initTreeData: function(treeDataArray) {
        var self = this;
        $.map(treeDataArray, function(treeData) {
            treeData.id = treeData.id + "";
            var treeNodeModel = Object.create(TreeNodeModel);
            treeNodeModel.initWithOption(treeData);
            self.pushTreeNodeModel(treeNodeModel);
            if(treeData.ChildNodes) {
                self.dataCache[treeData.id] = treeData.ChildNodes;
                self.initTreeData(treeData.ChildNodes);
            }
        });
    },

    updateData: function (data) {
        this.setState({
            data: data
        });
    },

    reset: function() {
        this.initVariables();
        this.setState({
            data: []
        });
        this.refreshComboText();
    },

    createItem: function(index, text) {
        var treeNodeData = this.state.data[index];
        var isSelected = this.selectedIDs.indexOf(treeNodeData.id) >= 0;
        var canSelect = this.isMultiSelection() || !this.isSelectLeafOnly() || !treeNodeData.hasChildren;
        var treeNodeModel = Object.create(TreeNodeModel);
        treeNodeModel.initWithOption(this.state.data[index]);
        this.pushTreeNodeModel(treeNodeModel);
        return <TreeNode isSelected={isSelected} canSelect={canSelect} ref={treeNodeData.id} key={treeNodeData.id} shownIndex={index} didSelectNode={this.didSelectNode} didExpandNode={this.didExpandNode} treeNodeModel={treeNodeModel} />;
    },

    collapse: function() {
        this.unbindTriggerEvent();
        $(ReactDOM.findDOMNode(this.refs["comboList"])).hide();
    },

    isMultiSelection: function() {
        return this.props.options.mutiSelection;
    },

    isSelectLeafOnly: function() {
        return this.props.options.selectLeafOnly;
    },

    isReturnFullPath: function() {
        return this.props.options.returnFullPath;
    },

    //选择节点
    didSelectNode: function(node) {
        var self = this;
        var treeNodeModel = node.getTreeNodeModel();
        if(this.isMultiSelection()) {
            //多选
            var valueArray = [];
            if(this.isSelectLeafOnly()) {
                //只返回叶子节点
                if(treeNodeModel.hasChildren) {
                    if(node.isSelected()) {
                        //取消所有子节点
                        var nodeID = treeNodeModel.getID();
                        this.checkUnSelectUp(treeNodeModel);
                        var resultSelectIDs = [];
                        $.map(this.selectedIDs, function(id) {
                            if(id.indexOf(nodeID + "-") !== 0) {
                                resultSelectIDs.push(id);
                            } else {
                                var unSelectNode = self.getComboList().refs[id];
                                if(unSelectNode) {
                                    unSelectNode.setSelected(false);
                                }
                            }
                        });
                        this.selectedIDs = resultSelectIDs;
                    } else {
                        //选中所有子节点
                        this.selectedIDs.push(treeNodeModel.getID());
                        this.checkSelectUp(treeNodeModel);
                        this.loadAllLeafNode(treeNodeModel, this.selectedIDs);
                    }
                } else {
                    if(node.isSelected()) {
                        this.checkUnSelectUp(treeNodeModel);
                    } else {
                        //选中,这时候需要判断是不是所有的叶子节点都选中了,是的话,需要把父节点也选中
                        this.selectedIDs.push(treeNodeModel.getID());
                        this.checkSelectUp(treeNodeModel);
                    }
                }
            } else {
                if(node.isSelected()) {
                    //取消选中
                    this.selectedIDs.splice(this.selectedIDs.indexOf(treeNodeModel.getID()), 1);
                } else {
                    //选中
                    this.selectedIDs.push(treeNodeModel.getID());
                }
            }
        } else {
            //单选
            this.selectedIDs = [];
            if(this.lastSelectNode && this.lastSelectNode != node) {
                this.lastSelectNode.setSelected(false);
            }
            if(!node.isSelected()) {
                this.selectedIDs.push(treeNodeModel.getID());
                this.lastSelectNode = node;
            } else {
                this.lastSelectNode = null;
            }
            this.collapse();
        }
        this.sortTreeNodeIDs(this.selectedIDs);
        node.setSelected(!node.isSelected());
        this.refreshComboText();
        this.fireWidgetValueChange();
        this.fireEvent("afteredit");
    },

    checkSelectUp: function(treeNodeModel) {
        var self = this;
        var parentNodeModel = this.getParentNodeModel(treeNodeModel);
        if(!parentNodeModel) {
            return ;
        }

        var childrenNodeModels = this.getChildrenNodeModels(parentNodeModel);
        var hasAllChildrenChosen = true;
        $.each(childrenNodeModels, function(index, childNodeModel) {
            if(self.selectedIDs.indexOf(childNodeModel.getID()) < 0) {
                hasAllChildrenChosen = false;
                return false;
            }
        });
        if(hasAllChildrenChosen) {
            //选中父亲节点
            this.selectedIDs.push(parentNodeModel.getID());
            if(this.getComboList() && this.getComboList().refs[parentNodeModel.getID()]) {
                this.getComboList().refs[parentNodeModel.getID()].setSelected(true);
            }
            this.checkSelectUp(parentNodeModel);
        }
    },

    checkUnSelectUp: function(treeNodeModel) {
        //取消选中,本身和所有父节点也要取消选中
        var self = this;
        var nodeIDArray = treeNodeModel.getID().split("-");
        var currentID = "";
        $.map(nodeIDArray, function(id) {
            currentID += ((currentID ? "-" : "") + id);
            if(currentID != treeNodeModel.getID()) {
                if(self.getComboList() && self.getComboList().refs[currentID]) {
                    self.getComboList().refs[currentID].setSelected(false);
                }
            }
            var index = self.selectedIDs.indexOf(currentID);
            if(index >= 0) {
                self.selectedIDs.splice(index, 1);
            }
        });
    },

    loadAllLeafNode: function(treeNodeModel, selectedIDs) {
        var self = this;
        if(!selectedIDs) {
            selectedIDs = [];
        }
        this.loadNode(treeNodeModel, function(jsonData, isLoadFromServer) {
            $.map(jsonData, function(treeNodeModelJson) {
                if(selectedIDs.indexOf(treeNodeModelJson.id) < 0) {
                    selectedIDs.push(treeNodeModelJson.id);
                }
                var childTreeNodeModel = Object.create(TreeNodeModel);
                childTreeNodeModel.initWithOption(treeNodeModelJson);
                if(isLoadFromServer) {
                    self.pushTreeNodeModel(childTreeNodeModel);
                } else {
                    if(self.getComboList() && self.getComboList().refs[treeNodeModelJson.id]) {
                        self.getComboList().refs[treeNodeModelJson.id].setSelected(true);
                    }
                }
                if(childTreeNodeModel.hasChildren) {
                    self.loadAllLeafNode(childTreeNodeModel, selectedIDs);
                }
            });

        });
    },

    //展开/收起节点
    didExpandNode: function(node) {
        var self = this;
        if(node.isExpand()) {
            self.removeDataAtNode(node);
            node.setExpand(false);
        } else {
            this.loadNode(node.getTreeNodeModel(), function(jsonNodeModels) {
                self.insertDataAtNode(jsonNodeModels, node);
                node.setExpand(true);
            });
        }
    },

    loadNode: function(treeNodeModel, callback) {
        var self = this;
        var cacheNodeModels = this.dataCache[treeNodeModel.getID()];
        if(!cacheNodeModels) {
            var dependence = {};
            $.map(this.getLinkedWidgets(), function(linkedWidget) {
                dependence["$" + linkedWidget.getWidgetName()] = linkedWidget.getValue();
            });
            var parentNodeModel = this.getParentNodeModel(treeNodeModel);
            BaseUtils.ajax({
                url: FRH5ReportConstants.servletURL,
                dataType: 'text',
                async: false,
                data: {
                    dependence: BaseUtils.jsonEncode(dependence),
                    startIndex: 0,
                    reload: true,
                    op: "widget",
                    cmd: "getdata",
                    widgetname: self.props.options.widgetName,
                    sessionID: self.props.options.sessionID,
                    id: treeNodeModel.getID(),
                    value: treeNodeModel.getValue(),
                    parentID: parentNodeModel == null ? null : parentNodeModel.getID(),
                    parent_values: this.getTreeNodeModelValuePath(treeNodeModel)
                },
                type: 'POST',
                success: function(responseObject) {
                    var jsonData = BaseUtils.jsonDecode(responseObject);

                    self.dataCache[treeNodeModel.getID()] = jsonData;

                    $.map(jsonData, function(treeNodeJson) {
                        if(treeNodeJson.ChildNodes) {
                            self.dataCache[treeNodeJson.id] = treeNodeJson.ChildNodes;
                            self.initTreeData(treeNodeJson.ChildNodes);
                        }
                    });
                    callback(jsonData, true);
                }
            });
        } else {
            callback(cacheNodeModels, false);
        }
    },

    afterLoadData: function(jsonData) {
        var self = this;
        $.map(jsonData, function(treeNodeJson) {
            if(treeNodeJson.ChildNodes) {
                self.dataCache[treeNodeJson.id] = treeNodeJson.ChildNodes;
                self.initTreeData(treeNodeJson.ChildNodes);
            }
        });
    },

    insertDataAtNode: function(data, node) {
        this.updateData(this.state.data.slice(0, node.getShownIndex() + 1).concat(data).concat(this.state.data.slice(node.getShownIndex() + 1)));
    },

    removeDataAtNode: function(node) {
        var treeNodeModel = node.getTreeNodeModel();
        var nodeID = treeNodeModel.getID();
        var resultData = [];
        //要删除node的所有子节点,也就是id以nodeID-开始的所有节点
        $.map(this.state.data, function(data) {
            if(data.id.indexOf(nodeID + "-") !== 0) {
                resultData.push(data);
            }
        });
        this.updateData(resultData);
    },

    getComboList: function() {
        return this.refs["comboList"];
    },

    getNodeAtShownIndex: function(index) {
        if(index >= this.state.data.length) {
            return null;
        }
        return this.getNodeModelById(this.state.data[index].id);
    },

    getNodeModelById: function(nodeID) {
        var result;
        $.each(this.treeNodeModels, function(index, treeNodeModel) {
            if(treeNodeModel.getID() == nodeID) {
                result = treeNodeModel;
                return false;
            }
        });
        return result;
    },

    getValue: function() {
        var self = this;
        var isReturnFullPath = this.isReturnFullPath();
        var result = [];
        $.map(this.selectedIDs, function(id) {
            var treeNodeModel = self.getTreeNodeModelByID(id);
            if(!self.isSelectLeafOnly() || !treeNodeModel.hasChildren) {
                if(isReturnFullPath) {
                    result.push(self.getTreeNodeModelValuePath(treeNodeModel, true));
                } else {
                    result.push(treeNodeModel.getValue());
                }
            }
        });
        return this.isMultiSelection() ? result : result.join(this.delimiter);
    },

    setValue: function(value) {
        this.initTreeData(value.data);
        this.setState({
            data: value.data
        });
    },

    getText: function() {
        var self = this;
        var result = [];
        $.map(this.selectedIDs, function(id) {
            var treeNodeModel = self.getTreeNodeModelByID(id);
            if(!self.isSelectLeafOnly() || !treeNodeModel.hasChildren) {
                result.push(treeNodeModel.getText());
            }
        });
        return result.join(this.delimiter);
    },

    removeNodeInArray: function(nodeArray, nodeWillRemove) {
        $.each(nodeArray, function(index, node) {
            if(node.getID() == nodeWillRemove.getID()) {
                nodeArray.splice(index, 1);
                return false;
            }
        });
    },

    sortTreeNodeIDs: function(treeNodeIDs) {
        treeNodeIDs.sort(function(a, b) {
            var idArray_a = a.split("-");
            var idArray_b = b.split("-");

            for(var i = 0, length = idArray_a.length; i < length; i ++) {
                if(!idArray_b[i]) {
                    return true;
                }
                if(idArray_a[i] === idArray_b[i]) {
                    continue ;
                }
                return idArray_a[i] > idArray_b[i];
            }
        });

    },

    getTreeNodeModelByID: function(id) {
        var result ;
        $.each(this.treeNodeModels, function(index, treeNodeModel) {
            if(treeNodeModel.getID() == id) {
                result = treeNodeModel;
                return false;
            }
        });
        return result;
    },

    refreshComboText: function() {
        $(this.refs["combo"]).text(this.getText());
    },

    pushTreeNodeModel: function(treeNodeModel) {
        if(this.treeNodeModelIDs.indexOf(treeNodeModel.getID()) < 0) {
            this.treeNodeModels.push(treeNodeModel);
            this.treeNodeModelIDs.push(treeNodeModel.getID());
        }
    },

    getParentNodeModel: function(treeNodeModel) {
        var nodeID = treeNodeModel.getID();
        if(nodeID.indexOf("-") < 0) {
            return null;
        }
        var parentNodeID = nodeID.substring(0, nodeID.lastIndexOf("-"));
        return this.getTreeNodeModelByID(parentNodeID);
    },

    getChildrenNodeModels: function(parentNodeModel) {
        var result = [];
        var nodeID = parentNodeModel.getID();
        $.each(this.treeNodeModels, function(index, treeNodeModel) {
            if((treeNodeModel.level - parentNodeModel.level === 1) && treeNodeModel.id.indexOf(parentNodeModel.id + "-") === 0) {
                result.push(treeNodeModel);
            }
        });
        return result;
    },

    getTreeNodeModelValuePath: function(treeNodeModel, isArray, isText) {
        var result = [isText ? treeNodeModel.getText() : treeNodeModel.getValue()];
        var parentTreeNodeModel = this.getParentNodeModel(treeNodeModel);
        while(parentTreeNodeModel != null) {
            result.unshift(isText ? parentTreeNodeModel.getText() : parentTreeNodeModel.getValue());
            parentTreeNodeModel = this.getParentNodeModel(parentTreeNodeModel);
        }
        return isArray ? result : result.join(",");
    },

    getTreeNodeModelValue: function(value) {
        var self = this;
        var result;
        var level = value.split(",").length - 1;
        $.each(this.treeNodeModels, function(index, treeNodeModel) {
            if((treeNodeModel.level === level) && ((self.getTreeNodeModelValuePath(treeNodeModel, false, false) == value) || (self.getTreeNodeModelValuePath(treeNodeModel, false, true) == value))) {
                result = treeNodeModel;
                return false;
            }
        });
        return result;
    }
});