(function() {
    var i18nText = function(text) {
        return text;
    };

    $.extend(String, {
        leftPad: function (val, size, ch) {
            var result = String(val);
            if (!ch) {
                ch = " ";
            }
            while (result.length < size) {
                result = ch + result;
            }
            return result.toString();
        }
    });

    $.extend(Date.prototype, {
        /**
         * 获取时区
         * @returns {String}
         */
        getTimezone: function () {
            return this.toString().replace(/^.* (?:\((.*)\)|([A-Z]{1,4})(?:[\-+][0-9]{4})?(?: -?\d+)?)$/, "$1$2").replace(/[^A-Z]/g, "");
        },

        /**
         * 获取每月的最后一天
         * @returns {Date}
         */
        getLastDateOfMonth: function () {
            return new Date(this.getFullYear(), this.getMonth(), this.getMonthDays());
        },

        /**
         * 获取该月总天数
         * @param {Number} month 有参数表示指定月份，无参数表示当前月份
         * @returns {*}
         */
        getMonthDays: function (month) {
            var MD = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
            var year = this.getFullYear();
            if (typeof month == "undefined") {
                month = this.getMonth();
            }
            if (((0 === (year % 4)) && ( (0 !== (year % 100)) || (0 === (year % 400)))) && month === 1) {
                return 29;
            } else {
                return MD[month];
            }
        },

        /**
         * 获取当前日期属于该月的第几周
         * @returns {Number}
         */
        getWeekNumber: function () {
            var d = new Date(this.getFullYear(), this.getMonth(), this.getDate(), 0, 0, 0);
            var DoW = d.getDay();
            d.setDate(d.getDate() - (DoW + 6) % 7 + 3);
            var ms = d.valueOf();
            d.setMonth(0);
            d.setDate(4);
            return Math.round((ms - d.valueOf()) / (7 * 864e5)) + 1;
        }

    });

    Date.patterns = {
        ISO8601Long: "Y-m-d H:i:s",
        ISO8601Short: "Y-m-d",
        ShortDate: "n/j/Y",
        LongDate: "l, F d, Y",
        FullDateTime: "l, F d, Y g:i:s A",
        MonthDay: "F d",
        ShortTime: "g:i A",
        LongTime: "g:i:s A",
        SortableDateTime: "Y-m-d\\TH:i:s",
        UniversalSortableDateTime: "Y-m-d H:i:sO",
        YearMonth: "F, Y"
    };
    Date.parseFunctions = {count: 0};
    Date.parseRegexes = [];
    Date.formatFunctions = {count: 0};

    Date._MN = [i18nText("FR-Base_January"),
        i18nText("FR-Base_February"),
        i18nText("FR-Base_March"),
        i18nText("FR-Base_April"),
        i18nText("FR-Base_May"),
        i18nText("FR-Base_June"),
        i18nText("FR-Base_July"),
        i18nText("FR-Base_August"),
        i18nText("FR-Base_September"),
        i18nText("FR-Base_October"),
        i18nText("FR-Base_November"),
        i18nText("FR-Base_December")];

    Date._SMN = [
        i18nText("FR-Base_Short-Jan"),
        i18nText("FR-Base_Short-Feb"),
        i18nText("FR-Base_Short-Mar"),
        i18nText("FR-Base_Short-Apr"),
        i18nText("FR-Base_Short-May"),
        i18nText("FR-Base_Short-Jun"),
        i18nText("FR-Base_Short-Jul"),
        i18nText("FR-Base_Short-Aug"),
        i18nText("FR-Base_Short-Sep"),
        i18nText("FR-Base_Short-Oct"),
        i18nText("FR-Base_Short-Nov"),
        i18nText("FR-Base_Short-Dec")
    ];

    Date._DN=[
        i18nText("FR-Base_Sche-Sunday"),
        i18nText("FR-Base_Sche-Monday"),
        i18nText("FR-Base_Sche-Tuesday"),
        i18nText("FR-Base_Sche-Wednesday"),
        i18nText("FR-Base_Sche-Thursday"),
        i18nText("FR-Base_Sche-Friday"),
        i18nText("FR-Base_Sche-Saturday")
    ];

    Date._SDN=[
        i18nText("FR-Base_Sun"),
        i18nText("FR-Base_Mon"),
        i18nText("FR-Base_Tue"),
        i18nText("FR-Base_Wed"),
        i18nText("FR-Base_Thu"),
        i18nText("FR-Base_Fri"),
        i18nText("FR-Base_Sat")
    ];


    Date.y2kYear = 50;


    Date.monthNumbers = {
        Jan: 0,
        Feb: 1,
        Mar: 2,
        Apr: 3,
        May: 4,
        Jun: 5,
        Jul: 6,
        Aug: 7,
        Sep: 8,
        Oct: 9,
        Nov: 10,
        Dec: 11};

    Date.getMonthNumber = function (name) {
        return Date.monthNumbers[name.substring(0, 1).toUpperCase() + name.substring(1, 3).toLowerCase()];
    };

    Date.parseDate = function (input, format) {
        if (Date.parseFunctions[format] == null) {
            Date.createParser(format);
        }
        var func = Date.parseFunctions[format];
        return Date[func](input);
    };

    Date.createParser = function (format) {
        var funcName = "parse" + Date.parseFunctions.count++;
        var regexNum = Date.parseRegexes.length;
        var currentGroup = 1;
        Date.parseFunctions[format] = funcName;

        var code = "Date." + funcName + " = function(input){\n"
            + "var y = -1, m = -1, d = -1, h = -1, i = -1, s = -1, ms = -1, o, z, u, v;\n"
            + "var d = new Date();\n"
            + "y = d.getFullYear();\n"
            + "m = d.getMonth();\n"
            + "d = d.getDate();\n"
            + "var results = input.match(Date.parseRegexes[" + regexNum + "]);\n"
            + "if (results && results.length > 0) {";
        var regex = "";

        var special = false;
        var ch = '';
        for (var i = 0; i < format.length; ++i) {
            ch = format.charAt(i);
            if (!special && ch == "\\") {
                special = true;
            }
            else if (special) {
                special = false;
                regex += String.escape(ch);
            }
            else {
                var obj = Date.formatCodeToRegex(ch, currentGroup);
                currentGroup += obj.g;
                regex += obj.s;
                if (obj.g && obj.c) {
                    code += obj.c;
                }
            }
        }

        code += "if (u)\n"
            + "{v = new Date(u * 1000);}" // give top priority to UNIX time
            + "else if (y >= 0 && m >= 0 && d > 0 && h >= 0 && i >= 0 && s >= 0 && ms >= 0)\n"
            + "{v = new Date(y, m, d, h, i, s, ms);}\n"
            + "else if (y >= 0 && m >= 0 && d > 0 && h >= 0 && i >= 0 && s >= 0)\n"
            + "{v = new Date(y, m, d, h, i, s);}\n"
            + "else if (y >= 0 && m >= 0 && d > 0 && h >= 0 && i >= 0)\n"
            + "{v = new Date(y, m, d, h, i);}\n"
            + "else if (y >= 0 && m >= 0 && d > 0 && h >= 0)\n"
            + "{v = new Date(y, m, d, h);}\n"
            + "else if (y >= 0 && m >= 0 && d > 0)\n"
            + "{v = new Date(y, m, d);}\n"
            + "else if (y >= 0 && m >= 0)\n"
            + "{v = new Date(y, m);}\n"
            + "else if (y >= 0)\n"
            + "{v = new Date(y);}\n"
            + "}return (v && (z || o))?\n" // favour UTC offset over GMT offset
            + "    (z ? v.add(Date.SECOND, (v.getTimezoneOffset() * 60) + (z*1)) :\n" // reset to UTC, then add offset
            + "        v.add(Date.HOUR, (v.getGMTOffset() / 100) + (o / -100))) : v\n" // reset to GMT, then add offset
            + ";}";

        Date.parseRegexes[regexNum] = new RegExp("^" + regex + "$", "i");
        eval(code);
    };

    Date.parseDateTime = function(str, fmt) {
        var today = new Date();
        var y = 0;
        var m = 0;
        var d = 1;
        //wei : 对于fmt为‘YYYYMM’或者‘YYYYMMdd’的格式，str的值为类似'201111'的形式，因为年月之间没有分隔符，所以正则表达式分割无效，导致bug7376。
        var a = str.split(/\W+/);
        if(fmt.toLowerCase() == '%y%x' || fmt.toLowerCase() == '%y%x%d') {
            var tempStr = a.join('');
            var yearlength = 4;
            var otherlength = 2;
            a[0] = tempStr.substring(0, yearlength);
            a[1] = tempStr.substring(yearlength, yearlength+otherlength);
            a[2] = tempStr.substring(yearlength+otherlength, yearlength+otherlength*2);
        }
        var b = fmt.match(/%./g);
        var i = 0, j = 0;
        var hr = 0;
        var min = 0;
        var sec = 0;
        for (i = 0; i < a.length; ++i) {
            if (!a[i]){
                continue;
            }
            switch (b[i]) {
                case "%d":
                case "%e":
                    d = parseInt(a[i], 10);
                    break;

                case "%X":
                    m = parseInt(a[i], 10) - 1;
                    break;
                case "%x":
                    m = parseInt(a[i], 10) - 1;
                    break;

                case "%Y":
                case "%y":
                    y = parseInt(a[i], 10);
                    (y < 100) && (y += (y > 29) ? 1900 : 2000);
                    break;

                case "%b":
                case "%B":
                    for (j = 0; j < 12; ++j) {
                        if (Date._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { m = j; break; }
                    }
                    break;

                case "%H":
                case "%I":
                case "%k":
                case "%l":
                    hr = parseInt(a[i], 10);
                    break;

                case "%P":
                case "%p":
                    if (/pm/i.test(a[i]) && hr < 12){
                        hr += 12;
                    }
                    else if (/am/i.test(a[i]) && hr >= 12){
                        hr -= 12;
                    }
                    break;

                case "%M":
                    min = parseInt(a[i], 10);
                case "%S":
                    sec = parseInt(a[i], 10);
                    break;
            }
        }
        if (isNaN(y)) {
            y = today.getFullYear();
        }
        if (isNaN(m)) {
            m = today.getMonth();
        }
        if (isNaN(d)) {
            d = today.getDate();
        }
        if (isNaN(hr)) {
            hr = today.getHours();
        }
        if (isNaN(min)) {
            min = today.getMinutes();
        }
        if (isNaN(sec)) {
            sec = today.getSeconds();
        }
        if (y !== 0){
            return new Date(y, m, d, hr, min, sec);
        }
        y = 0; m = -1; d = 0;
        for (i = 0; i < a.length; ++i) {
            if (a[i].search(/[a-zA-Z]+/) != -1) {
                var t = -1;
                for (j = 0; j < 12; ++j) {
                    if (Date._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { t = j; break; }
                }
                if (t != -1) {
                    if (m != -1) {
                        d = m+1;
                    }
                    m = t;
                }
            } else if (parseInt(a[i], 10) <= 12 && m == -1) {
                m = a[i]-1;
            } else if (parseInt(a[i], 10) > 31 && y === 0) {
                y = parseInt(a[i], 10);
                (y < 100) && (y += (y > 29) ? 1900 : 2000);
            } else if (d === 0) {
                d = a[i];
            }
        }
        if (y === 0){
            y = today.getFullYear();
        }
        if (m !== -1 && d !== 0){
            return new Date(y, m, d, hr, min, sec);
        }
        return today;
    };

    Date.formatCodeToRegex = function (character, currentGroup) {
        switch (character) {
            case "d":
                return {g: 1,
                    c: "d = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{2})"}; // day of month with leading zeroes (01 - 31)
            case "D":
                for (var a = [], i = 0; i < 7; a.push(Date._SDN[i]), ++i) {
                }
                ; // get L10n short day names
                return {g: 0,
                    c: null,
                    s: "(?:" + a.join("|") + ")"};
            case "j":
                return {g: 1,
                    c: "d = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{1,2})"}; // day of month without leading zeroes (1 - 31)
            case "l":
                return {g: 0,
                    c: null,
                    s: "(?:" + Date.dayNames.join("|") + ")"};
            case "N":
                return {g: 0,
                    c: null,
                    s: "[1-7]"}; // ISO-8601 day number (1 (monday) - 7 (sunday))
            case "S":
                return {g: 0,
                    c: null,
                    s: "(?:st|nd|rd|th)"};
            case "w":
                return {g: 0,
                    c: null,
                    s: "[0-6]"}; // javascript day number (0 (sunday) - 6 (saturday))
            case "z":
                return {g: 0,
                    c: null,
                    s: "(?:\\d{1,3}"}; // day of the year (0 - 364 (365 in leap years))
            case "W":
                return {g: 0,
                    c: null,
                    s: "(?:\\d{2})"}; // ISO-8601 week number (with leading zero)
            case "F":
                return {g: 1,
                    c: "m = parseInt(Date.getMonthNumber(results[" + currentGroup + "]), 10);\n", // get L10n month number
                    s: "(" + Date._MN.join("|") + ")"};
            case "m":
                return {g: 1,
                    c: "m = parseInt(results[" + currentGroup + "], 10) - 1;\n",
                    s: "(\\d{2})"}; // month number with leading zeros (01 - 12)
            case "M":
                for (var a = [], i = 0; i < 12; a.push(Date._SMN[i]), ++i) {
                }
                ; // get L10n short month names
                return {g: 1,
                    c: "m = parseInt(Date.getMonthNumber(results[" + currentGroup + "]), 10);\n", // get L10n month number
                    s: "(" + a.join("|") + ")"};
            case "n":
                return {g: 1,
                    c: "m = parseInt(results[" + currentGroup + "], 10) - 1;\n",
                    s: "(\\d{1,2})"}; // month number without leading zeros (1 - 12)
            case "t":
                return {g: 0,
                    c: null,
                    s: "(?:\\d{2})"}; // no. of days in the month (28 - 31)
            case "L":
                return {g: 0,
                    c: null,
                    s: "(?:1|0)"};
            case "o":
            case "Y":
                return {g: 1,
                    c: "y = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{4})"}; // 4-digit year
            case "y":
                return {g: 1,
                    c: "var ty = parseInt(results[" + currentGroup + "], 10);\n"
                        + "y = ty > Date.y2kYear ? 1900 + ty : 2000 + ty;\n",
                    s: "(\\d{1,2})"}; // 2-digit year
            case "a":
                return {g: 1,
                    c: "if (results[" + currentGroup + "] == 'am') {\n"
                        + "if (h == 12) { h = 0; }\n"
                        + "} else { if (h < 12) { h += 12; }}",
                    s: "(am|pm)"};
            case "A":
                return {g: 1,
                    c: "if (results[" + currentGroup + "] == 'AM') {\n"
                        + "if (h == 12) { h = 0; }\n"
                        + "} else { if (h < 12) { h += 12; }}",
                    s: "(AM|PM)"};
            case "g":
            case "G":
                return {g: 1,
                    c: "h = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{1,2})"}; // 24-hr format of an hour without leading zeroes (0 - 23)
            case "h":
            case "H":
                return {g: 1,
                    c: "h = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{2})"}; //  24-hr format of an hour with leading zeroes (00 - 23)
            case "i":
                return {g: 1,
                    c: "i = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{2})"}; // minutes with leading zeros (00 - 59)
            case "s":
                return {g: 1,
                    c: "s = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{2})"}; // seconds with leading zeros (00 - 59)
            case "u":
                return {g: 1,
                    c: "ms = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(\\d{3})"}; // milliseconds with leading zeros (000 - 999)
            case "O":
                return {g: 1,
                    c: [
                        "o = results[", currentGroup, "];\n",
                        "var sn = o.substring(0,1);\n", // get + / - sign
                        "var hr = o.substring(1,3)*1 + Math.floor(o.substring(3,5) / 60);\n", // get hours (performs minutes-to-hour conversion also, just in case)
                        "var mn = o.substring(3,5) % 60;\n", // get minutes
                        "o = ((-12 <= (hr*60 + mn)/60) && ((hr*60 + mn)/60 <= 14))?\n", // -12hrs <= GMT offset <= 14hrs
                        "    (sn + String.leftPad(hr, 2, 0) + String.leftPad(mn, 2, 0)) : null;\n"
                    ].join(""),
                    s: "([+\\-]\\d{4})"}; // GMT offset in hrs and mins
            case "P":
                return {g: 1,
                    c: [
                        "o = results[", currentGroup, "];\n",
                        "var sn = o.substring(0,1);\n", // get + / - sign
                        "var hr = o.substring(1,3)*1 + Math.floor(o.substring(4,6) / 60);\n", // get hours (performs minutes-to-hour conversion also, just in case)
                        "var mn = o.substring(4,6) % 60;\n", // get minutes
                        "o = ((-12 <= (hr*60 + mn)/60) && ((hr*60 + mn)/60 <= 14))?\n", // -12hrs <= GMT offset <= 14hrs
                        "    (sn + String.leftPad(hr, 2, 0) + String.leftPad(mn, 2, 0)) : null;\n"
                    ].join(""),
                    s: "([+\\-]\\d{2}:\\d{2})"}; // GMT offset in hrs and mins (with colon separator)
            case "T":
                return {g: 0,
                    c: null,
                    s: "[A-Z]{1,4}"}; // timezone abbrev. may be between 1 - 4 chars
            case "Z":
                return {g: 1,
                    c: "z = results[" + currentGroup + "] * 1;\n" // -43200 <= UTC offset <= 50400
                        + "z = (-43200 <= z && z <= 50400)? z : null;\n",
                    s: "([+\\-]?\\d{1,5})"}; // leading '+' sign is optional for UTC offset
            case "c":
                var df = Date.formatCodeToRegex, calc = [];
                var arr = [df("Y", 1), df("m", 2), df("d", 3), df("h", 4), df("i", 5), df("s", 6), df("P", 7)];
                for (var i = 0, l = arr.length; i < l; ++i) {
                    calc.push(arr[i].c);
                }
                return {g: 1,
                    c: calc.join(""),
                    s: arr[0].s + "-" + arr[1].s + "-" + arr[2].s + "T" + arr[3].s + ":" + arr[4].s + ":" + arr[5].s + arr[6].s};
            case "U":
                return {g: 1,
                    c: "u = parseInt(results[" + currentGroup + "], 10);\n",
                    s: "(-?\\d+)"}; // leading minus sign indicates seconds before UNIX epoch
            default:
                return {g: 0,
                    c: null,
                    s: character.replace(/([.*+?^$}{()|[\]\/\\])/g, "\\$1")};
        }
    };

    // safari setMonth is broken
    if (navigator.userAgent.indexOf("Safari") > -1) {
        Date.brokenSetMonth = Date.prototype.setMonth;
        Date.prototype.setMonth = function (num) {
            if (num <= -1) {
                var n = Math.ceil(-num);
                var back_year = Math.ceil(n / 12);
                var month = (n % 12) ? 12 - n % 12 : 0;
                this.setFullYear(this.getFullYear() - back_year);
                return Date.brokenSetMonth.call(this, month);
            } else {
                return Date.brokenSetMonth.apply(this, arguments);
            }
        };
    }
})();


module.exports = {
    compileJFmt : function (jfmt, date) {
        var str = jfmt.str, len = jfmt.len, ch = jfmt['char'];
        switch(ch){
            case 'E': //星期
                str = Date._DN[date.getDay()];
                break;
            case 'y': //年
                if(len <= 3){
                    str = (date.getFullYear()+'').slice(2,4);
                }else{
                    str = date.getFullYear();
                }
                break;
            case 'M': //月
                if(len > 2){
                    str = Date._MN[date.getMonth()];
                }else if(len < 2){
                    str = date.getMonth() + 1;
                }else{
                    str = String.leftPad(date.getMonth() + 1 +'',2,'0');
                }
                break;
            case 'd': //日
                if(len > 1){
                    str = String.leftPad(date.getDate()+'',2,'0');
                }else{
                    str = date.getDate();
                }
                break;
            case 'h': //时(12)
                var hour = date.getHours()%12;
                if(hour === 0){
                    hour = 12;
                }
                if(len > 1){
                    str = String.leftPad(hour +'',2,'0');
                }else{
                    str = hour;
                }
                break;
            case 'H': //时(24)
                if(len > 1){
                    str = String.leftPad(date.getHours()+'',2,'0');
                }else{
                    str = date.getHours();
                }
                break;
            case 'm':
                if(len > 1){
                    str = String.leftPad(date.getMinutes()+'',2,'0');
                }else{
                    str = date.getMinutes();
                }
                break;
            case 's':
                if(len > 1){
                    str = String.leftPad(date.getSeconds()+'',2,'0');
                }else{
                    str = date.getSeconds();
                }
                break;
            case 'a':
                str = date.getHours()<12?'am':'pm';
                break;
            case 'z':
                str = date.getTimezone();
                break;
            default:
                str = jfmt.str;
                break;
        }
        return str;
    },

    /**
     * 将Java提供的日期格式字符串装换为JS识别的日期格式字符串
     * @param fmt 日期格式
     * @returns {String}
     */
    parseFmt : function (fmt) {
        if (!fmt) {
            return '';
        }
        //日期
        fmt = String(fmt)
            //年
            .replace(/y{4,}/g, '%Y')//yyyy的时候替换为Y
            .replace(/y{2}/g, '%y')//yy的时候替换为y
            //月
            .replace(/M{4,}/g, '%b')//MMMM的时候替换为b，八
            .replace(/M{3}/g, '%B')//MMM的时候替换为M，八月
            .replace(/M{2}/g, '%X')//MM的时候替换为X，08
            .replace(/M{1}/g, '%x')//M的时候替换为x，8
            .replace(/a{1}/g, '%p')
        //天
        if (new RegExp('d{2,}', 'g').test(fmt)) {
            fmt = fmt.replace(/d{2,}/g, '%d');//dd的时候替换为d
        } else {
            fmt = fmt.replace(/d{1}/g, '%e');//d的时候替换为j
        }
        //时
        if (new RegExp('h{2,}', 'g').test(fmt)) {//12小时制
            fmt = fmt.replace(/h{2,}/g, '%I');
        } else {
            fmt = fmt.replace(/h{1}/g, '%I');
        }
        if (new RegExp('H{2,}', 'g').test(fmt)) {//24小时制
            fmt = fmt.replace(/H{2,}/g, '%H');
        } else {
            fmt = fmt.replace(/H{1}/g, '%H');
        }
        fmt = fmt.replace(/m{2,}/g, '%M')//分
            //秒
            .replace(/s{2,}/g, '%S')

        return fmt;
    },

    /**
     * 把字符串按照对应的格式转化成日期对象
     *
     *      @example
     *      var result = FR.str2Date('2013-12-12', 'yyyy-MM-dd');//Thu Dec 12 2013 00:00:00 GMT+0800
     *
     * @class FR.str2Date
     * @param str 字符串
     * @param format 日期格式
     * @returns {*}
     */
    str2Date : function (str, format) {
        if (typeof str != 'string' || typeof format != 'string') {
            return null;
        }
        var fmt = this.parseFmt(format);
        return Date.parseDateTime(str, fmt);
    },

    /**
     * 把日期对象按照指定格式转化成字符串
     *
     *      @example
     *      var date = new Date('Thu Dec 12 2013 00:00:00 GMT+0800');
     *      var result = FR.date2Str(date, 'yyyy-MM-dd');//2013-12-12
     *
     * @class FR.date2Str
     * @param date 日期
     * @param format 日期格式
     * @returns {String}
     */
    date2Str : function(date, format){
        if(!date) {
            return '';
        }
        // O(len(format))
        var len = format.length, result = '';
        if (len > 0) {
            var flagch = format.charAt(0), start = 0, str = flagch;
            for (var i = 1; i < len; i++) {
                var ch = format.charAt(i);
                if (flagch !== ch) {
                    result += this.compileJFmt({
                        'char': flagch,
                        'str': str,
                        'len': i - start
                    }, date);
                    flagch = ch;
                    start = i;
                    str = flagch;
                }else{
                    str +=ch;
                }
            }
            result += this.compileJFmt({
                'char': flagch,
                'str': str,
                'len': len - start
            }, date);
        }
        return result;
    }
}