var React = require("react");
var BaseUtils = require("./baseUtils");

var Private = {
    fitLayoutWidth: 0,
    fitLayoutHeight: 0,
    scaleWidth: 1,
    scaleHeight: 1,
    compInterval: 4,   //固定间隙
    compState: 0,   //0:充满展现区域;1:自适应原样缩放
    formPadding: 4, //form的左右边距
    //服务器传过来的宽高xy不是真实的数据，去掉添加的间隙，并且支持缩放
    initTrueDataAttribute: function(result, isAutoLayout) {
        this.fitLayoutWidth = result.fitLayoutWidth;
        this.fitLayoutHeight = result.fitLayoutHeight;
        var screenWidth = BaseUtils.getWindowWidth() - 2 * this.formPadding;
        var screenHeight = BaseUtils.getWindowHeight() - 2 * this.formPadding;
        this.compState = result.compState;
        if(!isAutoLayout) {
            this.scaleWidth = screenWidth / this.fitLayoutWidth;
            this.scaleHeight = this.isFill() ? screenHeight / this.fitLayoutHeight : this.scaleWidth;
        }
    },

    isFill: function() {
        return this.compState === 0;
    },

    calculateTrueData: function(item) {
        item.width = this.getTrueWidthOrHeight(item.width * this.scaleWidth, item.x, this.fitLayoutWidth * this.scaleWidth);
        item.height = this.getTrueWidthOrHeight(item.height * this.scaleHeight, item.y, this.fitLayoutHeight * this.scaleHeight);
        item.x = item.x * this.scaleWidth + (item.x === 0 ? 0 : this.compInterval / 2);
        item.y = item.y * this.scaleHeight + (item.y === 0 ? 0 : this.compInterval / 2);
    },

    getTrueWidthOrHeight: function(itemWidthOrHeight, itemXOrY, fitLayoutWidthOrHeight) {
        if(itemWidthOrHeight == fitLayoutWidthOrHeight) {//占据一行或一列
            return itemWidthOrHeight;
        } else if(itemXOrY == 0 || (fitLayoutWidthOrHeight == (itemXOrY + itemWidthOrHeight))) {//一行的第一个或者最后一个
            return itemWidthOrHeight - this.compInterval / 2;
        } else {//中间的
            return itemWidthOrHeight - this.compInterval;
        }
    }
}

module.exports = {
    getBorderLayoutCenterElements: function(options) {
        var result = {};
        if(options.type == "border") {
            //目前只有center
            $.map(options.items, function(item) {
                if(item.region == "center") {
                    result = item.el;
                }
            });
        }
        Private.initTrueDataAttribute(result, this.isAutoLayout(result.appRelayout));
        $.map(result.items, function (item) {
            Private.calculateTrueData(item);
        });
        result.sessionID = options.sessionID;
        result.fitLayoutWidth = result.fitLayoutWidth * Private.scaleWidth;
        result.fitLayoutHeight = result.fitLayoutHeight * Private.scaleHeight;
        return result;
    },

    isWidget: function(item) {
        return item.type != "title";
    },

    isTextArea: function(item) {
        return item.type == "textarea";
    },

    isChart: function(item) {
        return item.type == "title" && item.items[0].type == "chartwidget";
    },

    isElementCase: function(item) {
        return item.type == "title" && item.items[0].type == "elementcase";
    },

    isAutoLayout: function(appRelayout) {
        return device.mobile() && appRelayout;
    }
}