var React = require("react");
var ReactDOM = require("react-dom");
var PopList = require("../components/something/popList");
var BaseUtils = require("./baseUtils");
var ChartUtils = require("./chartUtils");
var BridgeUtils = require("./bridgeUtils")
var LoadingUtils = require("./loadingUtils");
var ReactDOM = require('react-dom');
var React = require("react");

module.exports = {
    createHyperLinkFunction: function (hyperLinks, sessionID, callback) {
        var self = this;
        var createSingleLinkFunction = function (hyperLink) {
            var hyperLinkUrl = "";
            switch (hyperLink.type) {
                case "web":
                    hyperLinkUrl = hyperLink.url;
                    break;
                case "report":
                    var type = hyperLink.reportlet.substr(hyperLink.reportlet.length - 3, 3) == "frm" ? "formlet" : "reportlet";
                    hyperLinkUrl = FRH5ReportConstants.serverURL + FRH5ReportConstants.servletURL + "?" + type + "=" + BaseUtils.cjkEncode(hyperLink.reportlet);
                    break;
                case "dynamic":
                    return function (e) {
                        BaseUtils.stopEvent(e);
                        $.map(hyperLink.parameters, function(key, index) {
                            hyperLink.parameters[BaseUtils.cjkDecode(index)] = BaseUtils.cjkDecode(key);
                        });
                        BaseUtils.ajax({
                            url: FRH5ReportConstants.servletURL,
                            dataType: 'text',
                            data: {
                                op: "fr_dialog",
                                cmd: "parameters_d",
                                sessionID: sessionID,
                                __parameters__:  hyperLink.parameters
                            },
                            type: "POST",
                            success: function (responseObject) {
                                if ($.isFunction(callback)) {
                                    callback({
                                        type: "dynamic",
                                        sessionID: sessionID
                                    });
                                }
                            }
                        });
                    }
                    break;
                case "javascript":
                    return function (e) {
                        BaseUtils.stopEvent(e);
                        var script = hyperLink.script;
                        script = "(function(e){var as=arguments; return FR.tc(function(){arguments=as;;return eval(" + hyperLink.script + ");}, this)}).apply()";
                        if (script.indexOf("\"") === 0 && script.lastIndexOf("\"") === script.length - 1) {
                            script = script.substring(1, script.length - 1);
                        }
                        new Function(unescape(script))();
                    }
                    break;
                case "chartRelate":
                    return function (e) {
                        BaseUtils.stopEvent(e);
                        var relateName = hyperLink.relateName;
                        var para = hyperLink.parameters || "";
                        var isCell = hyperLink.isCell;
                        var idPrefix = (isCell ? "Cells__" : "") + relateName;
                        self.chartRelate(idPrefix, para, sessionID);
                    }
                    break;
                case "widget":
                    return function(e) {
                        BaseUtils.stopEvent(e);
                        if ($.isFunction(callback)) {
                            callback({
                                type: "widget",
                                sessionID: sessionID,
                                related: hyperLink.related,
                                parameters: hyperLink.parameters
                            });
                        }
                    }
                    break ;
            }
            var parameters = hyperLink.parameters ? hyperLink.parameters : {};
            hyperLinkUrl = decodeURIComponent(hyperLinkUrl);
            if (hyperLinkUrl.indexOf("reportlet=") >= 0) {
                parameters["op"] = FRH5ReportConstants.op;
            }
            hyperLinkUrl = BaseUtils.addParameters2HyperLinkUrl(hyperLinkUrl, parameters);
            return function (e) {
                BaseUtils.stopEvent(e);
                if (hyperLinkUrl) {
                    if(self.isJumpHyperLink(hyperLink)) {
                        if(window.JSBridge) {
                            //如果是定义了jsbridge的app端,则交给app端处理,这边return
                            BridgeUtils.dealWithHyperLink(hyperLinkUrl, hyperLink.name);
                            return ;
                        }
                    }
                    var target = (hyperLink.target === "_self" ? "_self" : "_blank");
                    BaseUtils.openWindow(hyperLinkUrl, target);
                }
            };
        };

        var hyperLinksWithoutMenu = [];
        var hyperLinksWithMenu = [];
        $.map(hyperLinks, function(hyperLink, index) {
            if(self.isNoMenuHyperLink(hyperLink)) {
                hyperLinksWithoutMenu.push(hyperLink);
            } else {
                hyperLinksWithMenu.push(hyperLink);
            }
        });
        return function(e) {
            BaseUtils.stopEvent(e);
            $.map(hyperLinksWithoutMenu, function(hyperLinkWithoutMenu, index) {
                createSingleLinkFunction(hyperLinkWithoutMenu)(e);
            });
            if(hyperLinksWithMenu.length === 1) {
                LoadingUtils.removeMask();
                createSingleLinkFunction(hyperLinksWithMenu[0])(e);
            } else if(hyperLinksWithMenu.length > 1) {
                var hyperLinkItems = [];
                var PopList = require("../components/something/popList");
                hyperLinksWithMenu.map(function (hyperLink) {
                    hyperLinkItems.push({
                        text: hyperLink.name, onClick: function () {
                            LoadingUtils.removeMask();
                            createSingleLinkFunction(hyperLink)(e);
                        }
                    });
                });
                ReactDOM.render(
                    <PopList items={hyperLinkItems}></PopList>,
                    LoadingUtils.createMask()[0]
                );
            }
        }
    },

    //超链是不是会跳转到新页面
    isJumpHyperLink: function(hyperLink) {
        return hyperLink.type == "web" || hyperLink.type == "report";
    },

    //超链是不是直接触发,不需要弹框
    isNoMenuHyperLink: function(hyperLink) {
        return hyperLink.type == "widget";
    },

    chartRelate: function (idPrefix, para, sessionID) {
        var self = this;
        var vanCharts = ChartUtils.getChartStartWith(idPrefix);
        BaseUtils.ajax({
            type: 'GET',
            url: BaseUtils.addParameters2HyperLinkUrl(FRH5ReportConstants.servletURL + '?op=chartlink', para),
            data: {
                cmd: 'refresh_relate_data',
                sessionID: sessionID,
                chartID: vanCharts.dom.id,
                chartWidth: $(vanCharts.dom).width(),
                chartHeight: $(vanCharts.dom).height(),
                sheetIndex: 0,
                ecName: "",
                __time: new Date().getTime()
            },
            dataType: 'text',
            success: function (chartRelateJS) {
                var jsonData = BaseUtils.jsonDecode(chartRelateJS);
                //图表联动后会丢失超链的配置信息,已经告诉eason
                jsonData.relateChartList[0].chartAttr.plotOptions.click = vanCharts.plotOptionsClick;
                vanCharts.resize(jsonData.relateChartList[0].chartAttr);
            }
        });
    },

    getCompleteUrl: function (url, para, target) {
        var hyperlinkUrl;
        if (target == "_dialog") {
            hyperlinkUrl = ""; //移动端不支持dialog
        }
        if (BaseUtils.isNormalUrl(url)) {
            hyperlinkUrl = url;
        } else {
            hyperlinkUrl = FRH5ReportConstants.serverURL + FRH5ReportConstants.servletURL + (url.indexOf("?") < 0 ? "" : url.substr(url.indexOf("?")));
        }
        return encodeURI(hyperlinkUrl);
    },

    doHyperlinkByGet: function (url, para, target) {
        var options = arguments[0];
        if (typeof options === 'object') {
            url = options.url || '';
            para = options.para || {};
            target = options.target;
        }
        var url = this.getCompleteUrl(url, para, target);
        if (url == "") {
            return;
        }
        if(!para) {
            para = {};
        }
        if (url.indexOf("reportlet=") >= 0 || url.indexOf("formlet=") >= 0) {
            para.op = FRH5ReportConstants.op;
            url = BaseUtils.addParameters2HyperLinkUrl(url, para);
        }
        BaseUtils.openWindow(url, target);
    },

    doHyperlinkByPost: function (url, para, target) {
        //先和get一样处理,为了支持app中调用
        this.doHyperlinkByGet(url, para, target);

//        var options = arguments[0];
//        if (typeof options === 'object') {
//            url = options.url || '';
//            para = options.para || {};
//            target = options.target;
//        }
//        var url = this.getCompleteUrl(url, para, target)
//        if (url == "") {
//            return;
//        }
//        if(!para) {
//            para = {};
//        }
//        if (url.indexOf("reportlet=") >= 0 || url.indexOf("formlet=") >= 0) {
//            para.op = FRH5ReportConstants.op;
//        }
//
//        if (url) {
//            var submitForm = $("<form method='post' target='" + target + "' action='" + url + "'></form>").hide().appendTo($('body'));
//            for (var i in para) {
//                $("<input type=\"hidden\" name=\"" + i + "\" value=\"\"/>")
//                    .attr("value", BaseUtils.cjkEncode(BaseUtils.cjkDecode(para[i]))).appendTo(submitForm);
//            }
//            submitForm.submit();
//        }
    }
}