var React = require("react");
var ReactDOM = require("react-dom");
var BaseUtils = require("./baseUtils");
var LoadingUtils = require("./loadingUtils");
var ChartUtils = require("./chartUtils");
var HyperLinkUtils = require("./hyperLinkUtils");

(function register_g() {
    if (!window._g) {
        window._g = function (sessionID) {
            return {
                loadContentPane: function () {
                    var self = this;
                    $.ajax({
                        url: FRH5ReportConstants.servletURL,
                        dataType: 'text',
                        data: {
                            op: "page_content",
                            cmd: "json",
                            sessionID: sessionID,
                            pn: 1
                        },
                        type: "POST",
                        success: function (responseObject) {
                            var jsonData = BaseUtils.jsonDecode(responseObject);
                            window.contentPane.setState({
                                content: getReportTableOptions(jsonData)
                            });
                        }
                    });
                }
            }
        }
    }
})();

var Private = {
    _parseCss: function (json, css, zoomScale) {
        for (var key in json) {
            if (key == "color") {
                css["color"] = json[key];
            } else if (key == "size") {
                css["fontSize"] = Math.max(json[key] * zoomScale, 10);
            }
        }
    },

    openUrlThroughForm: function (url) {
        var submitForm = $("<form method='post' target='_blank' action='" + url + "'></form>").hide().appendTo($('body'));
        submitForm.submit();
    }
};

var getReportTableOptions = function (options) {
    var pageContent = options["pageContent"];
    var charts = options["charts"];
    var reportDetail = pageContent["detail"];
    var pageLayoutInfo = reportDetail[0]["pageLayoutInfo"];
    return {
        colWidths: pageLayoutInfo["colWidth"],
        rowHeights: pageLayoutInfo["rowHeight"],
        rows: reportDetail[0]["cellData"]["rows"],
        charts: charts,
        sessionID: options["sessionID"],
        reportWidth: BaseUtils.getTotalLength(pageLayoutInfo["colWidth"]),
        reportHeight: BaseUtils.getTotalLength(pageLayoutInfo["rowHeight"])
    };
};

module.exports = {
    createCssStyleFromCellStyle: function (reportStyle, zoomScale) {
        var cssStyle = {};
        if (reportStyle.font) {
            Private._parseCss(reportStyle.font, cssStyle, zoomScale);
        }
        if (reportStyle.background) {
            BaseUtils.analyseBackground(reportStyle.background, cssStyle);
        }
        if (reportStyle.border) {
            if (reportStyle.border["b"]) {
                cssStyle["borderBottomColor"] = reportStyle.border["b"]["c"];
            }
            if (reportStyle.border["l"]) {
                cssStyle["borderLeftColor"] = reportStyle.border["l"]["c"];
            }
            if (reportStyle.border["r"]) {
                cssStyle["borderRightColor"] = reportStyle.border["r"]["c"];
            }
            if (reportStyle.border["t"]) {
                cssStyle["borderTopColor"] = reportStyle.border["t"]["c"];
            }
        }
        if (reportStyle.shouldHide) {
            cssStyle["display"] = "none";
        }
        return cssStyle;
    },

    createCellValue: function (cellValue, sessionID, width, height, onDoHyperLink) {
        var self = this;
        if (!cellValue) {
            return null;
        }
        if (cellValue.type == "image") {
            //图片
            return (<div style={this.createImageStyle(BaseUtils.dealWithImageData(cellValue.value), cellValue.imageLayout, width, height)}></div>);
        } else if (cellValue.type == "simplechart") {
            //图表
            var Chart = require("../components/widget/chart");
            cellValue.items[0].onDoHyperLink = onDoHyperLink;
            return (<Chart key={cellValue.items[0].simpleChartInShowID} options={cellValue.items[0]} chartWidth={width} chartHeight={height}></Chart>);
        } else if (cellValue.type == "bias") {
            return (<img src={cellValue.imageData}></img>);
        }
    },

    createImageStyle: function (imageData, imageLayout, width, height, style) {
        if(!style) {
            style = {};
        }
        style["backgroundImage"] = BaseUtils.dealWithImageData(imageData);
        style["width"] = width;
        style["height"] = height;
        BaseUtils.analyseBackgroundLayout(imageLayout, imageData, style, width, height);
        return style;
    },

    getReportSize: function (options) {
        var pageContent = options["pageContent"];
        var reportDetail = pageContent["detail"];
        var pageLayoutInfo = reportDetail[0]["pageLayoutInfo"];
        return {
            width: BaseUtils.getTotalLength(pageLayoutInfo["colWidth"]),
            height: BaseUtils.getTotalLength(pageLayoutInfo["rowHeight"])
        };
    },

    getPageLayoutInfo: function (options) {
        var pageContent = options["pageContent"];
        var reportDetail = pageContent["detail"];
        return reportDetail[0]["pageLayoutInfo"];
    },

    hasFrozen: function (options) {
        var pageLayoutInfo = this.getPageLayoutInfo(options);
        return pageLayoutInfo.frozenRow > 0 || pageLayoutInfo.frozenColumn > 0;
    },

    getFrozenRow: function (options) {
        var pageLayoutInfo = this.getPageLayoutInfo(options);
        return pageLayoutInfo.frozenRow;
    },

    getFrozenHeight: function (options, frozenRow) {
        var frozenHeight = 0;
        for (var i = 0; i < frozenRow; i++) {
            frozenHeight += options["rowHeights"][i];
        }
        return frozenHeight;
    },

    getFrozenColumn: function (options) {
        var pageLayoutInfo = this.getPageLayoutInfo(options);
        return pageLayoutInfo.frozenColumn;
    },

    getFrozenWidth: function (options, frozenColumn) {
        var frozenWidth = 0;
        for (var i = 0; i < frozenColumn; i++) {
            frozenWidth += options["colWidths"][i];
        }
        return frozenWidth;
    },

    getFrozenReportTotalWidth: function (options) {
        var columnWidths = options["colWidths"];
        var result = 0;
        for (var index = 0, length = columnWidths.length; index < length; index++) {
            result += columnWidths[index];
        }
        return result;
    },

    getFrozenReportTotalHeight: function (options) {
        var rowHeights = options["rowHeights"];
        var result = 0;
        for (var index = 0, length = rowHeights.length; index < length; index++) {
            result += rowHeights[index];
        }
        return result;
    },

    getFrozenReportTableOptions: function (options, frozenRow, frozenColumn, position) {
        var charts = options["charts"];
        var colWidths = options["colWidths"];
        var rowHeights = options["rowHeights"];
        var rows = options["rows"];
        var rowStartIndex, rowEndIndex, colStartIndex, colEndIndex = -1;
        switch (position) {
            case "north":
                rowStartIndex = 0;
                rowEndIndex = frozenRow;
                colStartIndex = frozenColumn;
                colEndIndex = colWidths.length;
                break;
            case "west":
                rowStartIndex = frozenRow;
                rowEndIndex = rowHeights.length;
                colStartIndex = 0;
                colEndIndex = frozenColumn;
                break;
            case "corner":
                rowStartIndex = 0;
                rowEndIndex = frozenRow;
                colStartIndex = 0;
                colEndIndex = frozenColumn;
                break;
            case "center":
                rowStartIndex = frozenRow;
                rowEndIndex = rowHeights.length;
                colStartIndex = frozenColumn;
                colEndIndex = colWidths.length;
                break;
        }
        var frozenColWidths = [];
        if (colEndIndex > 0 && rowEndIndex > 0) {
            for (var index = 0; index < colStartIndex; index++) {
                frozenColWidths.push(0);    //当隐藏列处理
            }
            frozenColWidths = frozenColWidths.concat(colWidths.slice(colStartIndex, colEndIndex));
        }
        var frozenRowHeights = (colEndIndex <= 0 || rowEndIndex <= 0) ? [] : rowHeights.slice(rowStartIndex, rowEndIndex);
        var frozenRows = [];
        if (colEndIndex > 0 && rowEndIndex > 0) {
            var frozenRowsWithAllColumns = rows.slice(rowStartIndex, rowEndIndex);
            for (var i = 0, len = frozenRowsWithAllColumns.length; i < len; i++) {
                var temp = $.extend(true, {}, frozenRowsWithAllColumns[i]);
                temp["cells"] = this.collectRightCells4Frozen(temp["cells"], colStartIndex, colEndIndex);
                frozenRows.push(temp);
            }
        }
        var reportWidth = 0;
        var reportHeight = 0;
        $.map(frozenColWidths, function(width) {
            reportWidth += width;
        });
        $.map(frozenRowHeights, function(height) {
            reportHeight += height;
        });
        return {
            colWidths: frozenColWidths,
            rowHeights: frozenRowHeights,
            rows: frozenRows,
            charts: charts,
            sessionID: options["sessionID"],
            reportWidth: reportWidth,
            reportHeight: reportHeight
        }
    },

    collectRightCells4Frozen: function (cells, colStartIndex, colEndIndex) {
        var result = [];
        $.map(cells, function (cell) {
            if (cell.col >= colStartIndex && cell.col < colEndIndex) {
                result.push(cell);
            }
        });
        return result;
    },

    getReportTableOptions: function (options) {
        return getReportTableOptions(options);
    },

    loadElementCaseContent: function(sessionID, pageNumber, reportName, callback) {
        var self = this;
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
                dataType: "text",
                data: {
                    op: "fr_form",
                    cmd: "load_content_mobile",
                    sessionID: sessionID,
                    pageIndex: pageNumber,
                    widgetName: reportName
                },
                type: "POST",
                success: function(responseObject) {
                    var jsonData = BaseUtils.jsonDecode(responseObject);
                    if($.isFunction(callback)) {
                        callback(self.getReportTableOptions(jsonData));
                    }
                }
        });
    },

    loadReportContent: function (sessionID, pageNumber, callback) {
        var self = this;
        LoadingUtils.showLoading();
        BaseUtils.ajax({
            url: FRH5ReportConstants.servletURL,
            dataType: "text",
            data: {
                op: "page_content",
                cmd: "json",
                sessionID: sessionID,
                pn: pageNumber
            },
            type: "POST",
            success: function (responseObject) {
                LoadingUtils.hideLoading();
                var jsonData = BaseUtils.jsonDecode(responseObject);
                if ($.isFunction(callback)) {
                    callback(self.getReportTableOptions(jsonData));
                }
            },
            error: function() {
                LoadingUtils.hideLoading();
            }
        });
    },

    isNeedAppendBlankTD: function () {
        //Safari tr里如果没有td, 那么上一个含有td的tr高度会失效.
        return BaseUtils.isWebKit() && !BaseUtils.isChrome();
    },

    registerContentPane: function (contentPane) {
        window.contentPane = contentPane;
    }
}