/**
 * Created by joyxu on 2016/4/19.
 */
var React = require("react");
var BaseUtils = require("./baseUtils");

module.exports = {

    Private: {
        hasInit: false,
        widgetsToInitLink: [],
        refs: null,
        self: null
    },

    getWidgetByName: function(widgetName) {
        var widgetRef = this.Private.refs[widgetName.toUpperCase()];
        return widgetRef ? widgetRef.getWidget() : null;
    },

    collectWidgetValues: function() {
        var result = {};
        for(var widgetName in this.Private.refs) {
            var widget = this.getWidgetByName(widgetName);
            if(widget && $.isFunction(widget.getValue)) {
                result[widgetName] = widget.getValue();
            }
        }
        return result;
    },

    widgetsInitLink :function(refs){
        var Private = this.Private;
        Private.refs = refs;
        Private.self = this;
        Private.hasInit = true;
        Private.widgetsToInitLink.map(function(widget) {
            Private.self.initLink(widget);
        });
    },

    widgetBindForm: function (form) {
        for(var widgetName in this.Private.refs) {
            var widget = this.getWidgetByName(widgetName);
            if(widget) {
                if(!widget.options) {
                    widget.options = {};
                }
                widget.options.form = form;
            }
        }
    },

    initLink: function(widget) {
        var Private = this.Private;
        if(!Private.hasInit) {
            Private.widgetsToInitLink.push(widget);
            return ;
        }
        var options = widget.props.options;
        var dependencies = [];
        if(options["valueDependence"]) {
            dependencies = dependencies.concat(options["valueDependence"]);
        }
        if(options["dependence"]) {
            dependencies = dependencies.concat(options["dependence"]);
        }
        dependencies.map(function(value) {
            if(value == "$$$") {
                return ;
            }
            var widgetName = value;
            if(widgetName.indexOf("$") === 0) {
                widgetName = widgetName.substring(1);
            }
            var dependenceWidget = Private.self.getWidgetByName(widgetName);
            if(dependenceWidget) {
                dependenceWidget.pushLinkWidget(widget);
                widget.pushLinkedWidget(dependenceWidget);
            }
        });
    },

    fireWidgetValueChange: function(widget) {
        var Private = this.Private;
        var sessionID = widget.getSessionID();
        var linkWidgetNames = [];
        widget.getLinkWidgets().map(function(linkWidget) {
            var linkWidget = Private.self.getWidgetByName(linkWidget.props.options.widgetName);
            linkWidget.reset();
            linkWidgetNames.push(linkWidget.props.options.widgetName);
        });
        if(linkWidgetNames.length > 0) {
            linkWidgetNames = this.dealWithElementCaseAndChart(linkWidgetNames);
            BaseUtils.ajax({
                url: FRH5ReportConstants.servletURL,
                dataType:'text',
                data: {
                    op: "fr_form",
                    cmd: "form_getsource",
                    sessionID: sessionID,
                    __widgetname__: linkWidgetNames,
                    __parameters__: BaseUtils.jsonEncode(Private.self.collectWidgetValues())
                },
                type: "POST",
                success:function(responseObject) {
                    var jsonData = BaseUtils.jsonDecode(responseObject);
                    for(var widgetName in jsonData) {
                        var widget = Private.self.getWidgetByName(widgetName);
                        widget.setValue && widget.setValue(jsonData[widgetName]);
                    }
                }
            });
        }
    },

    //报表块被控件联动的情况
    dealWithElementCaseAndChart: function(linkWidgetNames) {
        var Private = this.Private;
        var normalWidgets = [];
        $.map(linkWidgetNames, function(widgetName, index) {
            var widget = Private.self.getWidgetByName(widgetName);
            if(widget && widget.props.options.type == "title") {
                widget.refreshWhenBeLinked(Private.self.collectWidgetValues());
            } else {
                normalWidgets.push(widgetName);
            }
        });
        return normalWidgets;
    },

    validateWidget: function() {
        var errorResult = [];
        for(var widgetName in this.Private.refs) {
            var widget = this.getWidgetByName(widgetName);
            if(widget && $.isFunction(widget.isValidate)) {
                if(!widget.isValidate()) {
                    errorResult.push({
                        widgetName: widgetName,
                        errorMessage: widget.getErrorMessage()
                    });
                }
            }
        }
        if(errorResult.length > 0) {
            this.showErrorMessage(errorResult);
            return false;
        }
        return true;
    },

    showErrorMessage: function(errorMessage) {
        var resultStr = "";
        $.map(errorMessage, function(message) {
            resultStr += (message.errorMessage + "\n")
        });
        alert(resultStr);   //todo 实现弹框组件
    }
};